C $run-col.for
C
      FUNCTION COLUNI(METHOD,ZS,RSGN,D,HS,AMU,ACOL,DET)
C 
c Vypocet odezvy detektoru na zareni prochazejici valcovym otvorem
c z bodoveho zarice s jednotkovou aktivitou (vzdaleneho o |rsgn| od osy
c valce, a o vzdalenost ZS od cela kolimatoru).
c
c METH=1   - view factor (bez korekce na blizkou vzdalenost)
c METH=2   - view factor (s korekci)
c METH=3   - 1R (bez korekce)
c METH=4   - 1R (s korekci na blizkost)
c METH=5   - 3R detektor o vetsim prumeru nez je prumer diry (bez korekce)
c METH=6   - 3R detektor o vetsim prumeru nez je prumer diry s korekci
c
c ZS,RSGN  - poloha bodoveho zdroje s jednotkovou aktivitou
c
c D   - prumer otvoru
c HS  - delka valcoveho otvoru
c AMU - soucinitel utlumu prostredi
c ACOL- soucinitel utlumu materialu kolimatoru
c det - prumer detektoru
c
      PARAMETER (PI=3.141)
      REAL L,LH,LH2,L1,L2,LM1,LM2,LC1,LC2
      R=ABS(RSGN)
      Z=AMAX1(ZS,1E-6)
      H=AMAX1(HS,1E-4)
      SELECT CASE(METHOD)
      CASE(1)
c
c View factor bez korekce
c
         R1=D/2
         R2=R1*(H+Z)/Z
         R12=R1*R1
         R22=R2*R2
         E=H*R/Z
         L=SQRT(Z**2+R**2)
         ZH=Z+H
         LH2=ZH**2+R**2
         LH=SQRT(LH2)
         IF(E.GE.R1+R2)THEN
            S=0
         ELSEIF(E.LE.R2-R1)THEN
            S=PI*R12
         ELSEIF(E.GE.SQRT(R22-R12))THEN
            B=SQRT(AMAX1(0.,4*R12*R22-(E**2-R12-R22)**2))/2
            BER1=B/(E*R1)
            BER2=B/(E*R2)
            S=R12*ASIN(BER1)+R22*ASIN(BER2)-B
         ELSE
            B=SQRT(AMAX1(0.,4*R12*R22-(E**2-R12-R22)**2))/2
            BER1=B/(E*R1)
            BER2=B/(E*R2)
            S=R12*(PI-ASIN(BER1))+R22*ASIN(BER2)-B
         ENDIF
         COLUNI=S*ZH*EXP(-AMU*L)/(4*PI*LH**3)
      CASE(2)
C
C View factor s korekci na blizkou vzdalenost
C
         R1=D/2
         R2=R1*(H+Z)/Z
         R12=R1*R1
         R22=R2*R2
         E=H*R/Z
         L=SQRT(Z**2+R**2)
         ZH=Z+H
         LH2=ZH**2+R**2
         LH=SQRT(LH2)
         IF(E.GE.R1+R2)THEN
            S=0
         ELSEIF(E.LE.R2-R1)THEN
            S=PI*R12
         ELSEIF(E.GE.SQRT(R22-R12))THEN
            B=SQRT(AMAX1(0.,4*R12*R22-(E**2-R12-R22)**2))/2
            BER1=B/(E*R1)
            BER2=B/(E*R2)
            S=R12*ASIN(BER1)+R22*ASIN(BER2)-B
         ELSE
            B=SQRT(AMAX1(0.,4*R12*R22-(E**2-R12-R22)**2))/2
            BER1=B/(E*R1)
            BER2=B/(E*R2)
            S=R12*(PI-ASIN(BER1))+R22*ASIN(BER2)-B
         ENDIF
         COLUNI=S*ZH*EXP(-AMU*L)/(2*PI*R12)*(1./LH-1./SQRT(LH2+R12))
      CASE(3)
C
c 1R bez korekce
C
         R1=D/2
         ZH=Z+H
         LH=SQRT(ZH**2+R**2)
         RLM=SQRT((R-R*H/ZH)**2+(H-ZH)**2)
         IF(R/ZH.LE.R1/H)THEN
            RLC=0
         ELSE
            RLC=SQRT((R1-R*H/ZH)**2+(H-ZH*R1/R)**2)
         ENDIF
         COLUNI=ZH*EXP(-AMU*RLM-ACOL*RLC)/(4*PI*LH**3)
      CASE(4)
C
c 1R s korekci
C
         R1=D/2
         ZH=Z+H
         LH2=ZH**2+R**2
         LH=SQRT(LH2)
         RLM=SQRT((R-R*H/ZH)**2+(H-ZH)**2)
         IF(R/ZH.LE.R1/H)THEN
            RLC=0
         ELSE
            RLC=SQRT((R1-R*H/ZH)**2+(H-ZH*R1/R)**2)
         ENDIF
         COLUNI=ZH*EXP(-AMU*RLM-ACOL*RLC)/2*
     /       (1./LH-1./SQRT(LH2+R12))
      CASE(5)
C
C 3R S ROZDILNYM PRUMEREM DETEKTORU BEZ KOREKCE
C
         R1=D/2
         R12=R1**2
         ZH=Z+H
         LH2=ZH**2+R**2
         LH=SQRT(LH2)
         RLM=SQRT((R-R*H/ZH)**2+(H-ZH)**2)
         IF(R/ZH.LE.R1/H)THEN
            RLC=0
         ELSE
            RLC=SQRT((R1-R*H/ZH)**2+(H-ZH*R1/R)**2)
         ENDIF
         COLUNI=ZH*EXP(-AMU*RLM-ACOL*RLC)/(4*PI*LH**3)
         BR=(DET+D)/4
         DS=PI*(DET**2-D**2)/8
         L1=SQRT((BR-R)**2+ZH**2)
         L2=SQRT((BR+R)**2+ZH**2)
         LM1=L1*(1-H/ZH)
         LM2=L2*(1-H/ZH)
         IF(R.LT.BR)THEN
           LC1=L1*AMIN1(H/ZH,(BR-D/2)/(BR-R))
         ELSE
           LC1=L1*H/ZH
         ENDIF
         LC2=L2*AMIN1(H/ZH,(BR-D/2)/(BR+R))
         COL=DS*ZH/(4*PI)*(EXP(-AMU*LM1-ACOL*LC1)/L1**3+
     /                               EXP(-AMU*LM2-ACOL*LC2)/L2**3)
         COLUNI=COLUNI+COL
      CASE(6)
C
C 3R S ROZDILNYM PRUMEREM DETEKTORU S KOREKCI
C 
         R1=D/2
         R12=R1**2
         ZH=Z+H
         LH2=ZH**2+R**2
         LH=SQRT(LH2)
         RLM=SQRT((R-R*H/ZH)**2+(H-ZH)**2)
         IF(R/ZH.LE.R1/H)THEN
            RLC=0
         ELSE
            RLC=SQRT((R1-R*H/ZH)**2+(H-ZH*R1/R)**2)
         ENDIF
         COLUNI=ZH*EXP(-AMU*RLM-ACOL*RLC)/2*
     /       (1./LH-1./SQRT(LH2+R12))
         BR=(DET+D)/4
         DS=PI*(DET**2-D**2)/8
         L1=SQRT((BR-R)**2+ZH**2)
         L2=SQRT((BR+R)**2+ZH**2)
         LM1=L1*(1-H/ZH)
         LM2=L2*(1-H/ZH)
         IF(R.LT.BR)THEN
           LC1=L1*AMIN1(H/ZH,(BR-D/2)/(BR-R))
         ELSE
           LC1=L1*H/ZH
         ENDIF
         LC2=L2*AMIN1(H/ZH,(BR-D/2)/(BR+R))
         COL=DS*ZH/(4*PI)*(EXP(-AMU*LM1-ACOL*LC1)/L1**3+
     /                               EXP(-AMU*LM2-ACOL*LC2)/L2**3)
         COLUNI=COLUNI+COL
      CASE(7)
C Kulovy detektor o prumeru D
         R1=D/2
         ZH=Z+H
         LH=SQRT(ZH**2+R**2)
         COLUNI=ZH*EXP(-AMU*LH)/(4*PI*LH**3)
      END SELECT
      END  

      SUBROUTINE ATRLEN(IG,X1,Y1,Z1,DIST)
C Vzdalenost bodu od cela detektoru
      INCLUDE '$FEM'
      X0=DETECT(IG,1)
      Y0=DETECT(IG,2)
      Z0=DETECT(IG,3)
      DIST=SQRT((X1-X0)**2+(Y1-Y0)**2+(Z1-Z0)**2)
      END

      SUBROUTINE ATILEN(IG,X1,Y1,Z1,XC,YC,ZC,A,B,H,ITYP,DIST)
C Totez co ATIDIS, ale souradnice detektoru IG jsou vybrany z databaze
      INCLUDE '$FEM'
      X0=DETECT(IG,1)
      Y0=DETECT(IG,2)
      Z0=DETECT(IG,3)
      CALL ATIDIS(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A,B,H,ITYP,DIST)
      END

      SUBROUTINE ATIDIS(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A,B,H,ITYP,DIST)
C Realizace funkce ALEN - delka paprsku v pruniku desky nebo valce
      DIMENSION T(9)
      LOGICAL SORT
      T=-1000
      RL=SQRT((X1-X0)**2+(Y1-Y0)**2+(Z1-Z0)**2)
      DIST=0.
      SELECT CASE(ITYP)
      CASE(1)
C DESKA X
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC-H/2,YC,ZC,A,B,1,T(1))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC+H/2,YC,ZC,A,B,1,T(2))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC-B/2,ZC,H,A,2,T(3))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC+B/2,ZC,H,A,2,T(4))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC-A/2,H,B,3,T(5))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC+A/2,H,B,3,T(6))
         N=6
      CASE(2)
C DESKA Y
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC-H/2,ZC,A,B,2,T(1))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC+H/2,ZC,A,B,2,T(2))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC-A/2,YC,ZC,B,H,1,T(3))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC+A/2,YC,ZC,B,H,1,T(4))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC-B/2,A,H,3,T(5))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC+B/2,A,H,3,T(6))
         N=6
      CASE(3)
C DESKA Z
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC-H/2,A,B,3,T(1))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC+H/2,A,B,3,T(2))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC-A/2,YC,ZC,H,B,1,T(3))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC+A/2,YC,ZC,H,B,1,T(4))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC-B/2,ZC,A,H,2,T(5))
         CALL ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC+B/2,ZC,A,H,2,T(6))
         N=6
      CASE(4)
C Valcovy plast X
         CALL ATICYL(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A,B,1,T(1),T(2))
         CALL ATICYL(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A-H,B,1,T(3),T(4))
         CALL ATIANU(X0,Y0,Z0,X1,Y1,Z1,XC-B/2,YC,ZC,A-H,A,1,T(5))
         CALL ATIANU(X0,Y0,Z0,X1,Y1,Z1,XC+B/2,YC,ZC,A-H,A,1,T(6))
         N=6
      CASE(5)
C Valcovy plast Y
         CALL ATICYL(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A,B,2,T(1),T(2))
         CALL ATICYL(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A-H,B,2,T(3),T(4))
         CALL ATIANU(X0,Y0,Z0,X1,Y1,Z1,XC,YC-B/2,ZC,A-H,A,2,T(5))
         CALL ATIANU(X0,Y0,Z0,X1,Y1,Z1,XC,YC+B/2,ZC,A-H,A,2,T(6))
         N=6
      CASE(6)
C Valcovy plast Z
         CALL ATICYL(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A,B,3,T(1),T(2))
         CALL ATICYL(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A-H,B,3,T(3),T(4))
         CALL ATIANU(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC-B/2,A-H,A,3,T(5))
         CALL ATIANU(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC+B/2,A-H,A,3,T(6))
         N=6
      CASE(7)
C Valec X
         CALL ATICYL(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A,B,1,T(1),T(2))
         CALL ATICIR(X0,Y0,Z0,X1,Y1,Z1,XC-B/2,YC,ZC,A,1,T(3))
         CALL ATICIR(X0,Y0,Z0,X1,Y1,Z1,XC+B/2,YC,ZC,A,1,T(4))
         N=4
      CASE(8)
C Valec Y
         CALL ATICYL(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A,B,2,T(1),T(2))
         CALL ATICIR(X0,Y0,Z0,X1,Y1,Z1,XC,YC-B/2,ZC,A,2,T(3))
         CALL ATICIR(X0,Y0,Z0,X1,Y1,Z1,XC,YC+B/2,ZC,A,2,T(4))
         N=4
      CASE(9)
C Valec Z
         CALL ATICYL(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A,B,3,T(1),T(2))
         CALL ATICIR(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC-B/2,A,3,T(3))
         CALL ATICIR(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC+B/2,A,3,T(4))
         N=4
      ENDSELECT
C Setrideni pruseciku T a vyrazeni zapornych
      SORT=.TRUE.
      DO WHILE(SORT)
         SORT=.FALSE.
         DO I=1,N-1
            IF(T(I).GT.T(I+1))THEN
                SORT=.TRUE.
                TT=T(I)
                T(I)=T(I+1)
                T(I+1)=TT
            ENDIF
         ENDDO
      ENDDO
      NP=1
      DO I=1,N
         IF(T(I).LT.0.)THEN
            NP=I+1
         ELSE
            T(I)=AMIN1(1.,T(I))
         ENDIF
      ENDDO
C Vyhodnoceni pruseciku T(NP),T(NP+1),...
      IF(NP.LE.N)THEN
         IF(T(NP+1).GT.0.)THEN
            DIST=T(NP+1)-T(NP)
            IF(T(NP+3).GT.0.)DIST=DIST+T(NP+3)-T(NP+2)
         ENDIF
      ENDIF
      DIST=DIST*RL
      END

      SUBROUTINE ATICYL(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,R,H,ITYP,T1,T2)
C
C prusecik usecky 0-1 s valcovou plochou o stredu C,polomeru R, a delce H.
C ITYP= 1,2,3
C t1<0 t2<0  primka jde mimo valec
C 0<t1<t2<1  dva pruseciky
C t1<0<t2<1  jeden prusecik
C t2<0<t1<1  jeden prusecik
C 1<t1<t2    usecka nedosahla k valci (zadny prusecik)
C 0<t1<1<t2  usecka konci nekde uvnitr valce (jeden prusecik)
C zaporne hodnoty (-1000) oznacuji zadny prusecik s primkou
C kladne hodnoty <=1 skutecny prusecik
C kladne hodnoty >1  prusecik pri prodlouzeni paprsku
C
      DX=X1-X0
      DY=Y1-Y0
      DZ=Z1-Z0
      X0C=X0-XC
      Y0C=Y0-YC
      Z0C=Z0-ZC
      DX2=DX**2
      DY2=DY**2
      DZ2=DZ**2
      R2=R**2
      T1=-1000
      T2=-1000
      SELECT CASE(ITYP)
      CASE (1)
C X-AXIS
         D=R2*(DY2+DZ2)-(DY*Z0C-DZ*Y0C)**2
         IF(D.GE.0.AND.DY2+DZ2.GT.0.)THEN
            S=SQRT(D)
            T1=(-DY*Y0C-DZ*Z0C-S)/(DY2+DZ2)
            T2=(-DY*Y0C-DZ*Z0C+S)/(DY2+DZ2)
C T1<T2
            XT1=X0+DX*T1
            XT2=X0+DX*T2
            XMI=XC-H/2
            XMA=XC+H/2
            IF(XT1.LT.XMI.OR.XT1.GT.XMA)T1=-1000
            IF(XT2.LT.XMI.OR.XT2.GT.XMA)T2=-1000
         ENDIF
      CASE (2)
C Y-AXIS
         D=R2*(DX2+DZ2)-(DX*Z0C-DZ*X0C)**2
         IF(D.GE.0.AND.DX2+DZ2.GT.0.)THEN
            S=SQRT(D)
            T1=(-DX*X0C-DZ*Z0C-S)/(DX2+DZ2)
            T2=(-DX*X0C-DZ*Z0C+S)/(DX2+DZ2)
C T1<T2
            YT1=Y0+DY*T1
            YT2=Y0+DY*T2
            YMI=YC-H/2
            YMA=YC+H/2
            IF(YT1.LT.YMI.OR.YT1.GT.YMA)T1=-1000
            IF(YT2.LT.YMI.OR.YT2.GT.YMA)T2=-1000
         ENDIF
      CASE (3)
C Z-AXIS
         D=R2*(DX2+DY2)-(DX*Y0C-DY*X0C)**2
         IF(D.GE.0.AND.DX2+DY2.GT.0.)THEN
            S=SQRT(D)
            T1=(-DX*X0C-DY*Y0C-S)/(DX2+DY2)
            T2=(-DX*X0C-DY*Y0C+S)/(DX2+DY2)
C T1<T2
            ZT1=Z0+DZ*T1
            ZT2=Z0+DZ*T2
            ZMI=ZC-H/2
            ZMA=ZC+H/2
            IF(ZT1.LT.ZMI.OR.ZT1.GT.ZMA)T1=-1000
            IF(ZT2.LT.ZMI.OR.ZT2.GT.ZMA)T2=-1000
         ENDIF
      ENDSELECT
      END

      SUBROUTINE ATIPLA(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,A,B,ITYP,T)
C 
C Prusecik usecky 0-1 s obdelnikovou deskou (AxB)
C
      T=-1000
      DX=X1-X0
      DY=Y1-Y0
      DZ=Z1-Z0
      SELECT CASE(ITYP)
      CASE(1)
         IF(DX.NE.0.)THEN
            T=(XC-X0)/DX
            YT=Y0+T*DY
            ZT=Z0+T*DZ
            YMA=YC+B/2
            YMI=YC-B/2
            ZMA=ZC+A/2
            ZMI=ZC-A/2
            IF(YT.LT.YMI.OR.YT.GT.YMA.OR.ZT.LT.ZMI.OR.ZT.GT.ZMA)T=-1000
         ENDIF
      CASE(2)
         IF(DY.NE.0.)THEN
            T=(YC-Y0)/DY
            XT=X0+T*DX
            ZT=Z0+T*DZ
            XMA=XC+A/2
            XMI=XC-A/2
            ZMA=ZC+B/2
            ZMI=ZC-B/2
            IF(XT.LT.XMI.OR.XT.GT.XMA.OR.ZT.LT.ZMI.OR.ZT.GT.ZMA)T=-1000
         ENDIF
      CASE(3)
         IF(DZ.NE.0.)THEN
            T=(ZC-Z0)/DZ
            XT=X0+T*DX
            YT=Y0+T*DY
            XMA=XC+A/2
            XMI=XC-A/2
            YMA=YC+B/2
            YMI=YC-B/2
            IF(XT.LT.XMI.OR.XT.GT.XMA.OR.YT.LT.YMI.OR.YT.GT.YMA)T=-1000
         ENDIF
      ENDSELECT
      END

      SUBROUTINE ATICIR(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,R,ITYP,T)
C 
C Prusecik usecky 0-1 s DISKEM (polomer R)
C
      DX=X1-X0
      DY=Y1-Y0
      DZ=Z1-Z0
      T=-1000
      SELECT CASE(ITYP)
      CASE(1)
         IF(DX.NE.0.)THEN
            T=(XC-X0)/DX
            YT=Y0+T*DY
            ZT=Z0+T*DZ
            RT=SQRT((YT-YC)**2+(ZT-ZC)**2)
            IF(RT.GT.R)T=-1000
         ENDIF
      CASE(2)
         IF(DY.NE.0.)THEN
            T=(YC-Y0)/DY
            XT=X0+T*DX
            ZT=Z0+T*DZ
            RT=SQRT((XT-XC)**2+(ZT-ZC)**2)
            IF(RT.GT.R)T=-1000
         ENDIF
      CASE(3)
         IF(DZ.NE.0.)THEN
            T=(ZC-Z0)/DZ
            YT=Y0+T*DY
            XT=X0+T*DX
            RT=SQRT((YT-YC)**2+(XT-XC)**2)
            IF(RT.GT.R)T=-1000
         ENDIF
      ENDSELECT
      END

      SUBROUTINE ATIANU(X0,Y0,Z0,X1,Y1,Z1,XC,YC,ZC,R1,R2,ITYP,T)
C 
C Prusecik usecky 0-1 s MEZIKRUZIM (polomer R1,R2)
C
      DX=X1-X0
      DY=Y1-Y0
      DZ=Z1-Z0
      T=-1000
      SELECT CASE(ITYP)
      CASE(1)
         IF(DX.NE.0.)THEN
            T=(XC-X0)/DX
            YT=Y0+T*DY
            ZT=Z0+T*DZ
            RT=SQRT((YT-YC)**2+(ZT-ZC)**2)
            IF(RT.LT.R1.OR.RT.GT.R2)T=-1000
         ENDIF
      CASE(2)
         IF(DY.NE.0.)THEN
            T=(YC-Y0)/DY
            XT=X0+T*DX
            ZT=Z0+T*DZ
            RT=SQRT((XT-XC)**2+(ZT-ZC)**2)
            IF(RT.LT.R1.OR.RT.GT.R2)T=-1000
         ENDIF
      CASE(3)
         IF(DZ.NE.0.)THEN
            T=(ZC-Z0)/DZ
            YT=Y0+T*DY
            XT=X0+T*DX
            RT=SQRT((YT-YC)**2+(XT-XC)**2)
            IF(RT.LT.R1.OR.RT.GT.R2)T=-1000
         ENDIF
       ENDSELECT
       END

