C $S27-KLOC
C FEM-WAFI - expanze vyvolana tokem pary
C        Konstantni tloustka vrstvy H 
C        Reseni Poissonovy rovnice pro tlak, prava strana je pouze funkci teploty (bez iteraci)
C
C procedura PRESSURE - lokalni matice T3 nebo T6 elementu

      SUBROUTINE PRESSURE(ICONTR,IE,NL,NUE,AL,BL)
C
C Poissonova rovnice pro rozlozeni tlaku v trojuhelnikovych elementech
C Volitelny pocet Gaussovych uzlu integrace T:1,3,4,7 Q:1,2,3
C
C Parametry elementu
C JEPAR($FLOW2D)=$EDISS,$ELAV,$EII,$EGTX,$EGTY,$EGTZ,$EVX,$EVY,$EVZ,$ESXX,$ESYY,$ESXY,$EMISE,$EFOUL,$EPOWER
C    $EMISE - hustota toku pary
C    $ELAV  - pomer ploch deformovaneho/puvodniho trojuhelniku
C    $EDISS - soucinitel odparu FI
C    $EII   - teplota
C    $ESXX  - hmotnost skrobu v elementu [kg]
C    $ESYY  - hmotnost volne vody v elementu
C    $ESXY  - hmotnost vazane vody v elementu
C Uzlove parametry
C JPU(MDOF)=$TEMP,$PRES,$VOLT,$CN,$CD,$CA,$VX,$VY,$VZ,$UX,$UY,$RZ,$PS,$PSX,$PSY,$PSZ,$OMG,$HT
C    $PRES   - tlak
C    $UX,$UY - posuvy
C    $VX,$VY - rychlosti
C    $RZ     - soucet vnitrnich uhlu
C
      INCLUDE '$FEMEQ'
      DIMENSION X(9),Y(9),AL(NL,NL),BL(NL)
      DIMENSION G1(16),G2(16),WT(16),F(9),FX(9),FY(9),P(9)
C AL-MATICE SOUSTAVY, AM-MATICE PERMEABILITY, 
C G1,G2-GAUSSOVY UZLY PRO TROJUHELNIK WT-VAHY GAUS. UZLU 
C F,FX,FY-BAZOVE FUNKCE A JEJICH DERIVACE
C
      IF(ICONTR.GE.0)THEN
C Stanoveni souradnic uzlu trojuhelnikoveho elementu, prictenim posuvu UX,UY
      L0=LUE(IE)
	PMEAN=0
      DO IU=1,NUE
        IND=IABS(IUE(L0+IU))
        LOCUX=LPU($UX)+IND
        LOCUY=LPU($UY)+IND
        X(IU)=XX(IND)+VAL(LOCUX,3)
        Y(IU)=YY(IND)+VAL(LOCUY,3)
        LOCP=LPU($PRES)+IND
C Restrikce tlaku 
        P(IU)=AMAX1(0.,AMIN1(100.,VAL(LOCP,3)))
	  PMEAN=PMEAN+P(IU)
      ENDDO
	PMEAN=PMEAN/NUE
C EGROUP,MPROP,RRCONS
      LEGRP =MAX0(1,IGROUP(IE))
      LPROP =MAX0(1,IMPROP(IE))
      LRCON =MAX0(1,IRCONS(IE))
C EGROUP 
C     NAMELE=JGROUP(LEGRP,$ENAME)
      NAMELE=$FLOW2D
      NGAUS =MAX0(1,JGROUP(LEGRP,$GAUSS))
C RRCONS 
C H   - nominalni tloustka, vzdalenost desek ($H)
C TW  - teplota steny                        ($TE)
      H   =RRCONS(LRCON,$H)
      TW  =RRCONS(LRCON,$TE)
C Parametry elementu IE teploty $EII a hmotnostni tok pary $EMISE
C FLUX  - hustota hmotnostniho toku pary ($EMISE)
C TEMP  - teplota elementu               ($EII)
C FI    - soucinitel odparu              ($EDISS)
C WSTAR - hmotnost skrobu                ($ESXX)
C WFREE - hmotnost volne vody            ($ESYY)
C WBOND - hmotnost vazane vody           ($ESXY)
      LFLUX =MAX0(1,LEPAR(NAMELE,$EMISE))
      LTEMP =MAX0(1,LEPAR(NAMELE,$EII))
      LFI   =MAX0(1,LEPAR(NAMELE,$EDISS))
      LSTAR =MAX0(1,LEPAR(NAMELE,$ESXX))
      LFREE =MAX0(1,LEPAR(NAMELE,$ESYY))
      LBOND =MAX0(1,LEPAR(NAMELE,$ESXY))
      FLUX  =EPAR(IE,LFLUX)
      TEMP  =EPAR(IE,LTEMP)
      FI    =EPAR(IE,LFI)
      WSTAR =EPAR(IE,LSTAR)
      WFREE =EPAR(IE,LFREE)
      WBOND =EPAR(IE,LBOND)
C Materialove parametry elementu pro teplotu TEMP
C VISC - viskozita 
C RKX  - tepelna vodivost
C REVA - vyparne teplo
C BETA - soucinitel expanze
      VISC=RMPROP(LPROP,$VISC)
      IF(JMPROP(LPROP,$VISC).NE.0)VISC=VISC*CURFUN(JMPROP(LPROP,$VISC))
      RKX =RMPROP(LPROP,$KX)
	IF(JMPROP(LPROP,$KX)  .NE.0)RKX =RKX *CURFUN(JMPROP(LPROP,$KX))
      BETA=RMPROP(LPROP,$BETA)
      IF(JMPROP(LPROP,$BETA).NE.0)BETA=BETA*CURFUN(JMPROP(LPROP,$BETA))
      REVA=RMPROP(LPROP,$HM)
      IF(JMPROP(LPROP,$HM)  .NE.0)REVA=REVA*CURFUN(JMPROP(LPROP,$HM))
      CP  =RMPROP(LPROP,$CP)
      IF(JMPROP(LPROP,$CP)  .NE.0)CP  =CP  *CURFUN(JMPROP(LPROP,$CP))

C Parametry Poissonovy rovnice
	RKK=H**3/(12.*VISC)
C Rozliseni tri fazi
	IF(TEMP.LT.100.)THEN
C 1.FAZE nulova prava strana, zadny zdroj pary
	  RB=0
        ISTAGE=1
      ELSEIF(WFREE.GT.0.)THEN
C 2.FAZE odpar volne vody
	  TSAMPLE=AMIN1(TEMP,0.99*TW)
	  RB=BETA*2*RKX*(TW-TSAMPLE)/(H*REVA)
        ISTAGE=2
      ELSE
C 3.FAZE vazana voda - nulova expanze
        RB=WBOND*FI
        ISTAGE=3
      ENDIF

C TROJUHELNIKOVY ELEMENT T3 NEBO T6
      CALL TGAUS(NGAUS,NGSS,G1,G2,WT)


      IF(ICONTR.EQ.0)THEN
C LOKALNI MATICE AL(VODIVOSTI)
        AL=0 
        BL=0
      ELSE
        POWER=0
      ENDIF

C============================
C     IF(IE.EQ.46)WRITE(2,100)ISTAGE,H,TEMP,beta,
C    /FI,WSTAR,WFREE,WBOND
C100	FORMAT(/' sub PRESSURE IE=46 stage=',i2/' H=',F6.3,
C    /' T=',F7.1,' beta=',e10.3,' FI=',E10.3/
C    /' MS=',E10.3,' MF=',E10.3,' MB=',E10.3)
C============================


      GAUSS: DO IG=1,NGSS
        CALL FDFT(NUE,X,Y,G1(IG),G2(IG),F,FX,FY,DET)
	  E=DET*WT(IG)
        E1=E*RKK
C Prava strana je umerna hustote toku pary, ale zavisi take na porozite elementu EPS, tloustce H
	  E2=E*RB
        IF(ICONTR.EQ.0)THEN
C Reseni soustavy
          DO I=1,NUE
	      BL(I)=BL(I)+E2*F(I)
            DO J=1,NUE
              AL(I,J)=AL(I,J)+E1*(FX(I)*FX(J)+FY(I)*FY(J))   
            ENDDO
          ENDDO
        ELSE
C Postprocessing. Vypocet gradientu tlaku
          PSI=H**2/(12*VISC)
          PX=0
          PY=0
          DO I=1,NUE
             PX=PX+FX(I)*P(I)
             PY=PY+FY(I)*P(I)
          ENDDO
          VX=-PX*PSI
	    VY=-PY*PSI
C VX,VY lokalni rychlosti, je treba pripocist uzlum elementu v zavislosti na vnitrnim uhlu
          DO IU=1,3
	       I1=MOD(IU+1,3)+1
	       I2=IU
	       I3=MOD(IU,3)+1
C PTANGLE-Uhel mezi dvema navazujicimi orientovanymi useckami 1-2 a 2-3.
C Vystupni hodnota je v uhel v radianech
             ANGLE=ABS(PTANGLE(X(I1),Y(I1),X(I2),Y(I2),X(I3),Y(I3),IER))
             IND=IABS(IUE(L0+IU))
             LOCVX=LPU($VX)+IND
	       LOCVY=LPU($VY)+IND
	       LOCRZ=LPU($RZ)+IND
             VAL(LOCVX,3)=VAL(LOCVX,3)+VX*ANGLE
             VAL(LOCVY,3)=VAL(LOCVY,3)+VY*ANGLE
             VAL(LOCRZ,3)=VAL(LOCRZ,3)+ANGLE
          ENDDO
        ENDIF
      ENDDO GAUSS
      ENDIF
      END

