C $S28-KLOC
C FEM-WAFI	integralni model entalpickych bilanci. 
C         Pro vypocet teplot jsou pouzity vyhradne entalpicke bilance.
C         Tlak neovlivnuje teploty, ale jen expanzi.
C
C Procedury
C MFLUX  - pro kazdy element vyhodnotu entalpicke a hmotnostni bilance, spocita nove teploty.
C EXPAND - inktementace Ux,Uy ve vsech uzlech elementu
C WALLINI- pocatecni podminky, vypocet hmostnosti skrobu, volne a vazane vody v jednotlicych elementech
C BC     - uplatneni pokutove funkce pro tlak PRES v prihranicni zone
C WMATCH - korespondence se siti konecnych elementu
C TPLATE - teplotni pole v desce metodou siti (ADI)

      SUBROUTINE MFLUX(IE)
C Vypocet hmotnostniho toku pary z elementu

C Parametry elementu
C JEPAR($FLOW2D)=$EDISS,$ELAV,$EII,$EGTX,$EGTY,$EGTZ,$EVX,$EVY,$EVZ,$ESXX,$ESYY,$ESXY,$EMISE,$EFOUL,$EPOWER
C AREA  - pomer ploch deformovaneho/puvodniho trojuhelniku ($ELAV)
C FLUX  - hustota hmotnostniho toku pary ($EMISE)
C TEMP  - teplota elementu               ($EII)
C FI    - soucinitel odparu 3.stupne     ($EDISS) 
C WSTAR - hmotnost skrobu                ($ESXX)
C WFREE - hmotnost volne vody            ($ESYY)
C WBOND - hmotnost vazane vody           ($ESXY)

C Uzlove parametry
C JPU(MDOF)=$TEMP,$PRES,$VOLT,$CN,$CD,$CA,$VX,$VY,$VZ,$UX,$UY,$RZ,$PS,$PSX,$PSY,$PSZ,$OMG,$HT
C    $PRES   - tlak
C    $UX,$UY - posuvy
C    $VX,$VY - rychlosti
C    $RZ     - soucet vnitrnich uhlu

C RRCONS 
C H   - nominalni tloustka, vzdalenost desek ($H)
C TW  - teplota steny                        ($TE)
C 

C Materialove parametry elementu pro teplotu TEMP
C VISC - viskozita 
C RKX  - tepelna vodivost
C REVA - vyparne teplo
C BETA - soucinitel expanze
C CP   - merna tepelna kapacita
C PSI  - prutokovy koeficient pary  ($DN)

      INCLUDE '$FEMEQ'
	INCLUDE '$WGRID'
	DIMENSION X(3),Y(3),XNEW(3),YNEW(3)
	NAMELE=$FLOW2D
      LEGRP =MAX0(1,IGROUP(IE))
      LPROP =MAX0(1,IMPROP(IE))
      LRCON =MAX0(1,IRCONS(IE))

      H   =RRCONS(LRCON,$H)
C      TW  =RRCONS(LRCON,$TE)
C Teploty steny je treba pocitat z matice teplot desky

      LFLUX =MAX0(1,LEPAR(NAMELE,$EMISE))
      LTEMP =MAX0(1,LEPAR(NAMELE,$EII))
      LFI   =MAX0(1,LEPAR(NAMELE,$EDISS))
      LSTAR =MAX0(1,LEPAR(NAMELE,$ESXX))
      LFREE =MAX0(1,LEPAR(NAMELE,$ESYY))
      LBOND =MAX0(1,LEPAR(NAMELE,$ESXY))
      LAREA =MAX0(1,LEPAR(NAMELE,$ELAV))
      FLUX  =EPAR(IE,LFLUX)
      TEMP  =EPAR(IE,LTEMP)
      FI    =EPAR(IE,LFI)
      WSTAR =EPAR(IE,LSTAR)
      WFREE =EPAR(IE,LFREE)
      WBOND =EPAR(IE,LBOND)
	WMASS =WSTAR+WFREE+WBOND

      VISC=RMPROP(LPROP,$VISC)
      IF(JMPROP(LPROP,$VISC).NE.0)VISC=VISC*CURFUN(JMPROP(LPROP,$VISC))
      RKX =RMPROP(LPROP,$KX)
	IF(JMPROP(LPROP,$KX)  .NE.0)RKX =RKX *CURFUN(JMPROP(LPROP,$KX))
      BETA=RMPROP(LPROP,$BETA)
      IF(JMPROP(LPROP,$BETA).NE.0)BETA=BETA*CURFUN(JMPROP(LPROP,$BETA))
      REVA=RMPROP(LPROP,$HM)
      IF(JMPROP(LPROP,$HM)  .NE.0)REVA=REVA*CURFUN(JMPROP(LPROP,$HM))
      CP=RMPROP(LPROP,$CP)
      IF(JMPROP(LPROP,$CP)  .NE.0)CP  =CP  *CURFUN(JMPROP(LPROP,$CP))
      PSI=RMPROP(LPROP,$DN)
      IF(JMPROP(LPROP,$DN)  .NE.0)PSI =PSI *CURFUN(JMPROP(LPROP,$DN))

      L0=LUE(IE)
	NUE=3
	PMEAN=0
	XMEAN=0
	YMEAN=0
      DO IU=1,NUE
        IND=IABS(IUE(L0+IU))
        LOCUX=LPU($UX)+IND
        LOCUY=LPU($UY)+IND
	  LOCP=LPU($PRES)+IND
	  X(IU)=XX(IND)
	  Y(IU)=YY(IND)
        XNEW(IU)=XX(IND)+VAL(LOCUX,3)
        YNEW(IU)=YY(IND)+VAL(LOCUY,3)
	  XMEAN=XMEAN+XNEW(IU)
	  YMEAN=YMEAN+YNEW(IU)
        PMEAN=PMEAN+VAL(LOCP,3)
      ENDDO
	XMEAN=XMEAN/NUE
	YMEAN=YMEAN/NUE
C Zjisti uzel site, ktery odpovida tezisti elementu
C	COMMON /WGRID/DIMX,DIMY,NDIMX,NDIMY,
C     /   WHG(100,100),IHG(100,100),TNW(100,100),TOW(100,100),
C     /   IRISE,TWMIN,TWMAX,HPLATE,QPLATE,ATOP,ASIDE,
C     /   RLSTEEL,RHSTEEL,CPSTEEL
      DX=DIMX/(NDIMX-1)
	DY=DIMY/(NDIMY-1)
	IX=XMEAN/DX+1
	IY=YMEAN/DY+1
	IX=MAX0(1,MIN0(100,IX))
	IY=MAX0(1,MIN0(100,IY))
	TW=TOW(IX,IY)

	PMEAN=PMEAN/NUE
C Restrikce tlaku na 100 bar
      PMEAN=AMAX1(1.,AMIN1(100.,PMEAN))
C Vypocet ploch noveho a puvodniho trojuhelniku
      AOLD=ABS(Y(1)*(X(2)-X(3))+Y(2)*(X(3)-X(1))+Y(3)*(X(1)-X(2)))
      ANEW=ABS(YNEW(1)*(XNEW(2)-XNEW(3))+YNEW(2)*(XNEW(3)-XNEW(1))+
     /      YNEW(3)*(XNEW(1)-XNEW(2)))
C      EPAR(IE,LAREA)=ANEW/AOLD
	EPAR(IE,LAREA)=ANEW/2
C Rozliseni tri fazi
      ISTAGE=0
	IF(TEMP.LT.100.)THEN
C 1.FAZE implicitni metoda
        AA=2*DTIME*ANEW*RKX/(H*CP*WMASS)
	  TNEW=(TEMP+AA*TW)/(1+AA)
	  EPAR(IE,LTEMP)=TNEW
	  ISTAGE=1
      ELSEIF(WFREE.GT.0.)THEN
C 2.FAZE odpar volne vody
        AST=2*RKX*ANEW*DTIME/(WMASS*CP*H)
	  BST=ANEW*PSI*REVA*DTIME/(WMASS*CP)
	  T4=AMAX1(1.,(TEMP/100)**4)
	  TNEW=(TEMP+AST*TW+BST*(1+3*T4))/(1+AST+BST*4*T4/TEMP)
	  TNEW=AMAX1(100.,TNEW)
	  T4=AMAX1(1.,(TNEW/100)**4)
	  WFREE=AMAX1(0.,WFREE-DTIME*PSI*ANEW*(T4-1))
	  EPAR(IE,LTEMP)=TNEW
	  EPAR(IE,LFI)  =ANEW*PSI*(T4-1.)/WBOND
	  EPAR(IE,LFREE)=WFREE
	  ISTAGE=2
      ELSE
C 3.FAZE vazana voda 
        AST=2*RKX*ANEW*DTIME/(WMASS*CP*H)
	  BST=REVA*FI*WBOND*DTIME/(CP*WMASS)
	  TNEW=(TEMP+AST*TW-BST)/(1.+AST)
        WBOND=WBOND-DTIME*FI*WBOND
	  EPAR(IE,LTEMP)=TNEW
	  EPAR(IE,LBOND)=WBOND
	  ISTAGE=3
      ENDIF

C======================
C     IF(IE.EQ.46)WRITE(2,100)ISTAGE,H,TEMP,VISC,RKX,BETA,REVA,PSI,
C    /CP,FI,WSTAR,WFREE,WBOND
C00	FORMAT(/' MFLUX ISTAGE=',I2,' H=',F6.3,' T=',F7.1,
C    /' VISC=',E10.3,
C    /' KX=',E10.3,' BETA=',E10.3,' R=',E10.3,' psi=',e10.3,
C    /' CP=',E10.3,' FI=',E10.3/
C    /' MS=',E10.3,' MF=',E10.3,' MB=',E10.3)
C=======================
      END


      SUBROUTINE EXPAND
C Expanze site (vypocet $UX,$UY)
      INCLUDE '$WGRID'
      INCLUDE '$FEMLOC'
	DO IND=1,ND
        LOCVX=LPU($VX)+IND
        LOCVY=LPU($VY)+IND
        LOCRZ=LPU($RZ)+IND
        ANGLE=VAL(LOCRZ,3)
        VX=VAL(LOCVX,3)/ANGLE
        VY=VAL(LOCVY,3)/ANGLE
C Inkrement posuvu
        LOCUX=LPU($UX)+IND
        LOCUY=LPU($UY)+IND
	  DTX=VAL(LOCUX,3)+VX*DTIME
	  DTY=VAL(LOCUY,3)+VY*DTIME
	  DTX=AMAX1(0.,AMIN1(DIMX,DTX+XX(IND)))-XX(IND)
	  DTY=AMAX1(0.,AMIN1(DIMY,DTY+YY(IND)))-YY(IND)
        VAL(LOCUX,3)=DTX
        VAL(LOCUY,3)=DTY
        VAL(LOCUX,2)=DTX
        VAL(LOCUY,2)=DTY
	ENDDO  
      END

	SUBROUTINE WALLINI
C Vynulovani stenove mrizky a uzlovych parametru (posuv)
C	COMMON /WGRID/DIMX,DIMY,NDIMX,NDIMY,
C     /   WHG(100,100),IHG(100,100),TNW(100,100),TOW(100,100),
C     /   IRISE,TWMIN,TWMAX,HPLATE,QPLATE,ATOP,ASIDE,
C     /   RLSTEEL,RHSTEEL,CPSTEEL
      INCLUDE '$FEMLOC'
      INCLUDE '$WGRID'
	DIMENSION X(3),Y(3)
C GRID
	WHG=0.
	IHG=0
	TOW=(TWMIN+TWMAX)*.5
	TNW=TOW
	IRISE=1
	DO IND=1,ND
        LOCUX=LPU($UX)+IND
        LOCUY=LPU($UY)+IND
	  LOCPRESS=LPU($PRES)+IND
        VAL(LOCUX,3)=0
        VAL(LOCUY,3)=0
        VAL(LOCUX,2)=0
        VAL(LOCUY,2)=0
	  VAL(LOCPRESS,3)=0
	  VAL(LOCPRESS,2)=0
	  VAL(LOCPRESS,1)=0
	ENDDO  
C Pocatecni parametry elementu (hmotnosti WSTAR,WFREE,WBOND)
	NAMELE=$FLOW2D
      LFLUX =MAX0(1,LEPAR(NAMELE,$EMISE))
      LTEMP =MAX0(1,LEPAR(NAMELE,$EII))
      LFI   =MAX0(1,LEPAR(NAMELE,$EDISS))
      LSTAR =MAX0(1,LEPAR(NAMELE,$ESXX))
      LFREE =MAX0(1,LEPAR(NAMELE,$ESYY))
      LBOND =MAX0(1,LEPAR(NAMELE,$ESXY))
      LAREA =MAX0(1,LEPAR(NAMELE,$ELAV))

	DO IE=1,NE
        LRCON =MAX0(1,IRCONS(IE))
        H     =RRCONS(LRCON,$H)
        L0=LUE(IE)
	  NUE=3
        DO IU=1,NUE
          IND=IABS(IUE(L0+IU))
          X(IU)=XX(IND)
	    Y(IU)=YY(IND)
        ENDDO
C Vypocet ploch puvodniho trojuhelniku
        A=ABS(Y(1)*(X(2)-X(3))+Y(2)*(X(3)-X(1))+Y(3)*(X(1)-X(2)))
	  WMASS=A*H*1160
	  OMEGA=0.4
	  Z=0.44
	  WFREE=WMASS*(1.-OMEGA)/(1+Z)
	  WBOND=Z*WFREE
	  WSTAR=WMASS*OMEGA
C Pocatecni podminky 
        EPAR(IE,LFLUX)=0
        EPAR(IE,LTEMP)=20
        EPAR(IE,LFI)=0
        EPAR(IE,LSTAR)=WSTAR
        EPAR(IE,LFREE)=WFREE
        EPAR(IE,LBOND)=WBOND
      ENDDO
C konec WALLINI
      END


	SUBROUTINE BC
C Okrajove podminky pro tlak
      INCLUDE '$FEMLOC'
      INCLUDE '$WGRID'
C Vynulovani uzlovych parametru VX,VY a UHLU
	DO IND=1,ND
        LOCVX=LPU($VX)+IND
        LOCVY=LPU($VY)+IND
        LOCRZ=LPU($RZ)+IND
        VAL(LOCVX,3)=0.
        VAL(LOCVY,3)=0.
        VAL(LOCRZ,3)=0.
        LOCP=LPU($PRES)+IND
        IBND=IPU(LOCP)
	  IF(IBND.LT.0)THEN
C Hranice. tlak pokutova funkce
          LOCUX=LPU($UX)+IND
          LOCUY=LPU($UY)+IND
          X=XX(IND)+VAL(LOCUX,3)
          Y=YY(IND)+VAL(LOCUY,3)
	    DISMIN=AMIN1(DIMX,DIMY)*.01
	    DISTY=AMIN1(Y,DIMY-Y)
	    DISTX=AMIN1(X,DIMX-X)
	    DISTXY=AMIN1(DISTX,DISTY)
	    IF(DISTXY.GT.DISMIN)THEN
            VAL(LOCP,1)=0.
          ELSE
            VAL(LOCP,1)=2E-1*(DISMIN-DISTXY)/DISMIN
	    ENDIF
        ENDIF
      ENDDO
	END

	SUBROUTINE WMATCH
C koresponnce elementu a gridu IHG
      INCLUDE '$FEMLOC'
      INCLUDE '$WGRID'
C	COMMON /WGRID/DIMX,DIMY,NDIMX,NDIMY,
C     /   WHG(100,100),IHG(100,100),TNW(100,100),TOW(100,100),
C     /   IRISE,TWMIN,TWMAX,HPLATE,QPLATE,ATOP,ASIDE,
C     /   RLSTEEL,RHSTEEL,CPSTEEL
      DIMENSION X(3),Y(3)
	IHG=0
	DX=DIMX/(NDIMX-1)
	DY=DIMY/(NDIMY-1)
	DO IE=1,NE
        L0=LUE(IE)
	  NUE=3
        DO IU=1,NUE
          IND=IABS(IUE(L0+IU))
          LOCUX=LPU($UX)+IND
          LOCUY=LPU($UY)+IND
          X(IU)=XX(IND)+VAL(LOCUX,3)
          Y(IU)=YY(IND)+VAL(LOCUY,3)
        ENDDO
        DO I=1,NDIMX
	    XQ=(I-0.5)*DX
	    DO J=1,NDIMY
	      YQ=(J-0.5)*DY
C Lezi bod XQ,YQ uvnitr trojuhelniku X,Y?
            CALL EINS2D3(X,Y,XQ,YQ,IN)
	      IF(IN.EQ.1)IHG(I,J)=IE
          ENDDO
        ENDDO
      ENDDO
	END


      SUBROUTINE TPLATE(ITIME)
C Teplotni pole v desce metodou siti. ADI s pulkroky DTIME/2
      INCLUDE '$FEMLOC'
      INCLUDE '$WGRID'
	DIMENSION A(100),B(100),C(100),R(100),U(100)
	CHARACTER*10 FILE
C	COMMON /WGRID/DIMX,DIMY,NDIMX,NDIMY,
C     /   WHG(100,100),IHG(100,100),TNW(100,100),TOW(100,100),
C     /   IRISE,TWMIN,TWMAX,HPLATE,QPLATE,ATOP,ASIDE,
C     /   RLSTEEL,RHSTEEL,CPSTEEL
C   TOGH - stare teplotni pole,
C   TNW - nove teplotni pole
C   IHG  - index korespondujici elementu (nebo nula, kdyz je tam prazdny prostor)
C
C Zjisteni prumerne teploty a toho, zda je topeni zapnuto (IRISE=1) nebo vypnuto

      TAVER=0
	DO I=1,NDIMX
	DO J=1,NDIMY
	  TAVER=TAVER+TOW(I,J)
	ENDDO
	ENDDO
	TAVER=TAVER/(NDIMX*NDIMY)
	IF(TAVER.GT.TWMAX)THEN
	   IRISE=0
      ELSEIF(TAVER.LT.TWMIN)THEN
	   IRISE=1
	ENDIF
	IF(IRISE.EQ.1)THEN
	   Q=QPLATE/(DIMX*DIMY*HPLATE)
	ELSE
	   Q=0
	ENDIF

	WRITE(2,'('' ITIME='',I3,'' Q='',E10.3,'' TAVER='',F10.1,
     /'' ASIDE='',F10.1,'' ATOP='',F10.1,'' ASAMPLE='',F10.1)')
     /	ITIME,Q,TAVER,ASIDE,ATOP,2/0.004

	TE=20
	DX=DIMX/(NDIMX-1)
	DY=DIMY/(NDIMY-1)
	DX2=DX*DX
	DY2=DY*DY
      B1=2*RHSTEEL*CPSTEEL/DTIME
      B2X=2*RLSTEEL/DX2
      B2Y=2*RLSTEEL/DY2
	B3=ATOP/HPLATE

C X-implicitni pulkrok y=2,3,...,ny-1
C ===============================================================
      DO IY=2,NDIMY-1
	DO IX=2,NDIMX-1
	  IE=IHG(IX,IY)
	  IF(IE.EQ.0)THEN
	    RKX=0
	    TSAMPLE=TAVER
	    H=0.004
	    B4=0
        ELSE
C Zakladni parametry - skrob a tloustka sterbiny
	    NAMELE=$FLOW2D
          LPROP  =MAX0(1,IMPROP(IE))
          LRCON  =MAX0(1,IRCONS(IE))
          H      =RRCONS(LRCON,$H)
          LTEMP  =MAX0(1,LEPAR(NAMELE,$EII))
          TSAMPLE=EPAR(IE,LTEMP)
C TEMP systemova promenna - teplotni zavislost RKX
	    TEMP   =TSAMPLE
          RKX    =RMPROP(LPROP,$KX)
 	    IF(JMPROP(LPROP,$KX).NE.0)RKX=RKX*CURFUN(JMPROP(LPROP,$KX))
	    B4=RKX/(H*HPLATE)
        ENDIF
C Koeficienty tridiagonalni soustavy 
        A(IX)=-B2X/2
	  C(IX)=A(IX)
	  B(IX)=B1+B2X+B3+B4
        R(IX)=B1*TOW(IX,IY)+
     /        B2Y/2*(TOW(IX,IY-1)-2*TOW(IX,IY)+TOW(IX,IY+1))+
     /        Q+B3*TE+B4*TSAMPLE
	ENDDO
C Okrajove podminky pro IX=1 a NDIMX
      A(1)=0
	B(1)=ASIDE+RLSTEEL/DX
	C(1)=-RLSTEEL/DX
	R(1)=ASIDE*TE
	A(NDIMX)=-RLSTEEL/DX
	B(NDIMX)=ASIDE+RLSTEEL/DX
	C(NDIMX)=0.
	R(NDIMX)=ASIDE*TE
C Reseni tridiagonalni soustavy SUBROUTINE TRIDAG(A,B,C,R,U,N)
      CALL TRIDAG(A,B,C,R,U,NDIMX)
	DO I=1,NDIMX
	  TNW(I,IY)=U(I)
      ENDDO
      ENDDO
C Prepsani starych teplot
      DO IY=2,NDIMY-1
	DO IX=1,NDIMX
	   TOW(IX,IY)=TNW(IX,IY)
      ENDDO
	ENDDO
C Dopocitani teplot na hranici IY=1 a NDIMY jen z okrajovych podminek
      DO IX=1,NDIMX
	   TOW(IX,1)=(RLSTEEL/DY*TNW(IX,2)+ASIDE*TE)/(ASIDE+RLSTEEL/DY)
	   TOW(IX,NDIMY)=(RLSTEEL/DY*TNW(IX,NDIMY-1)+ASIDE*TE)/
     /                  (ASIDE+RLSTEEL/DY)
	ENDDO

C Y-implicitni pulkrok X=2,3,...,NX-1
C ===============================================================
      DO IX=2,NDIMX-1
	DO IY=2,NDIMY-1
	  IE=IHG(IX,IY)
	  IF(IE.EQ.0)THEN
	    RKX=0
	    TSAMPLE=TAVER
	    H=0.004
	    B4=0
        ELSE
C Zakladni parametry - skrob a tloustka sterbiny
	    NAMELE=$FLOW2D
          LPROP  =MAX0(1,IMPROP(IE))
          LRCON  =MAX0(1,IRCONS(IE))
          H      =RRCONS(LRCON,$H)
          LTEMP  =MAX0(1,LEPAR(NAMELE,$EII))
          TSAMPLE=EPAR(IE,LTEMP)
	    TEMP   =TSAMPLE
          RKX    =RMPROP(LPROP,$KX)
 	    IF(JMPROP(LPROP,$KX).NE.0)RKX=RKX*CURFUN(JMPROP(LPROP,$KX))
	    B4=RKX/(H*HPLATE)
        ENDIF
C Koeficienty tridiagonalni soustavy 
        A(IY)=-B2Y/2
	  C(IY)=A(IY)
	  B(IY)=B1+B2Y+B3+B4
        R(IY)=B1*TOW(IX,IY)+
     /        B2X/2*(TOW(IX-1,IY)-2*TOW(IX,IY)+TOW(IX+1,IY))+
     /        Q+B3*TE+B4*TSAMPLE
	ENDDO
C Okrajove podminky pro IY=1 a NDIMY
      A(1)=0
	B(1)=ASIDE+RLSTEEL/DY
	C(1)=-RLSTEEL/DY
	R(1)=ASIDE*TE
	A(NDIMY)=-RLSTEEL/DY
	B(NDIMY)=ASIDE+RLSTEEL/DY
	C(NDIMY)=0.
	R(NDIMY)=ASIDE*TE
C Reseni tridiagonalni soustavy SUBROUTINE TRIDAG(A,B,C,R,U,N)
      CALL TRIDAG(A,B,C,R,U,NDIMY)
	DO I=1,NDIMY
	  TNW(IX,I)=U(I)
      ENDDO
      ENDDO
C Prepsani starych teplot
      DO IY=1,NDIMY
	DO IX=2,NDIMX-1
	   TOW(IX,IY)=TNW(IX,IY)
      ENDDO
	ENDDO
C Dopocitani teplot na hranici IX=1 a NDIMX jen z okrajovych podminek
      DO IY=1,NDIMY
	   TOW(1,IY)=(RLSTEEL/DX*TNW(2,IY)+ASIDE*TE)/(ASIDE+RLSTEEL/DX)
	   TOW(NDIMX,IY)=(RLSTEEL/DX*TNW(NDIMX-1,IY)+ASIDE*TE)/
     /                  (ASIDE+RLSTEEL/DX)
	ENDDO
C Zapis vypocteneho teplotniho pole do souboru
      FILE='$T0000.OUT'
      CALL FILINUM(FILE,ITIME)
      OPEN(30,FILE=FILE)
      WRITE(30,'(2I6,E12.3,'' NDIMX,NDIMY,Q'')')NDIMX,NDIMY,Q
	DO IY=1,NDIMY
        WRITE(30,'(100ES11.3)')(TOW(IX,IY),IX=1,NDIMX)
      ENDDO
      CLOSE(30)
      END

      SUBROUTINE FILINUM(FILE,NUMBER)
C
C Vloz cislo NUMBER do textu FILE pred tecku.
C
      CHARACTER*(*) FILE
      IDOT=INDEX(FILE,'.')
      IF(IDOT.GT.1)THEN
        IF(NUMBER.LE.9)THEN
          WRITE(FILE(IDOT-1:IDOT-1),'(I1)')NUMBER
        ELSEIF(NUMBER.LE.99.AND.IDOT.GT.2)THEN
          WRITE(FILE(IDOT-2:IDOT-1),'(I2)')NUMBER
        ELSEIF(NUMBER.LE.999.AND.IDOT.GT.3)THEN
          WRITE(FILE(IDOT-3:IDOT-1),'(I3)')NUMBER
        ELSEIF(NUMBER.LE.9999.AND.IDOT.GT.4)THEN
          WRITE(FILE(IDOT-4:IDOT-1),'(I4)')NUMBER
        ENDIF
      ENDIF
      END
