      SUBROUTINE AHELP
C
C METODA KONECNYCH PRVKU
C-------------------------
C DATOVE STRUKTURY:
C 1. MATICE KONEKTIVITY SE NAHRAZUJE VEKTOREM KONEKTIVITY IUE(*) A
C    VEKTOREM UKAZOVATEK DO TOHOTO VEKTORU LUE(*). TO UMOZNUJE, ABY
C    BYLY VE VEKTORU KONEKTIVITY UKLADANY ELEMENTY S RUZNYMI POCTY UZLU.
C    POCET UZLU ELEMENTU IE JE MUE(IE), A INDEX I-TEHO UZLU
C    ELEMENTU IE JE IUE(LUE(IE)+I). VEKTOR LUE MUSI MIT DELKU NE+1.
C 2. INFORMACE O UZLECH JSOU VE VEKTORECH UKAZOVATEK LPU(*), KTERE
C    UDAVAJI POZICI UZLOVYCH PARAMETRU VE VEKTROU NEZNAMYCH X(*) A
C    VE VEKTORU STATUSU UZLOVYCH PARAMETRU IPU(*).
C    LPU(ND)+IP JE INDEX IP-TEHO PARAMETRU UZLU CISLO ND.
C    POCET UZLOVYCH PARAMETRU UZLU ND JE MND(ND).
C    ZAPORNE HODNOTY STAVOVEHO VEKTORU PARAMETRU IPU<0 OZNACUJI PARAMETR
C    FIXOVANY JAKO SILNA OKRAJOVA PODMINKA,IPU>=0 NEZNAME.
C    |IPU|>1000000 OZNACUJE PASIVOVANY UZLOVY PARAMETR,
C    KTERY NENI PRI RESENI SOUSTAVY ROVNIC UVAZOVAN.
C
C ALGORITMY:
C    Reseni uloh MKP je zalozeno na procedure FEFRON, ktera zajisti sestaveni
C elementu IE=1,2,...,NE a reseni soustavy rovnic A.X=F, podrobneji
C     ( Suma[As + Al] ).X = F + Suma[Fs + Fl]
C kde As jsou lokalni matice elementu (integrace pres oblast S, Intg[Ni,x.Nj,x ds])
C a Al lokalni matice (integrace pres hranici L, Intg[alfa.Ni.Nj dl]), coz
C odpovida okrajovym podminkam tretiho druhu. F jsou bodove zdroje
C (osamele sily, zdroje tepla), Fs plosne zdroje tepla nebo zatizeni vlastni vahou
C vyjadrene integraqly Intg[Q.Ni ds], Fl odpovida spojitemu zdroji nebo zatizeni
C tlakem na hranici L, tj. integralum Intg[p.Ni dl].
C    Pro vektor reseni X(*) i pro vektor prave strany F(*) pouziva FEFRON jediny
C spolecny vektor V(*). Na vstupu do FEFRON obsahuje V(*) uzlove hodnoty predepsanych
C silnych okrajovych podminek (status parametru IPU<0) nebo hodnoty bodoveho
C zatizeni F (status parametru IPU>=0), vsechny ostatni prvky musi byt nulove.
C Na vystupu FEFRON jsou tyto hodnoty V(*) prepsany resenim (samozrejme s vyjimkou
C silnych okrajovych podminek).
C    Procedura FEFRON vola v kazdem elementu uzivatelskou proceduru FEL, ktera
C musi spocitat matici As+Al a vektroy Fs+Fl. Pro jejich vypocet dostava k dispozici
C jedinou informaci, index elementu IE. Z vektoru konektivity IUE(*) musi stanovit
C indexy uzlu elementu ND, a z vektoru LPU(ND) status jednotlivych uzlovych
C parametru IPU(LPU(ND)+IP). Zatimco FEFRON rozlisuje jen zaporne a nezaporne
C hodnoty IPU (a ignoruje pasivovane parametry |IPU|>1000000), jsou konkretni
C hodnoty IPU pro proceduru FEL vyznamne, a dle nich se rozhoduje, zda ma pocitat
C Al,Fs,Fl a jaka transformacni funkce zatizeni nebo okrajovych podminek se
C ma pouzit.
C
C    Napriklad v programu COSMOS.FOR je procedura FEFRON vyuzivana nasledujicim zpusobem.
C Pri reseni se pouzivaji dva vektory Vo(*) a V(*) se stejnym vyznamem prvku urcenym
C vektorem IPU(*). Vektor Vo(*) je pripravovan preprocesorem (procedury NFCR, MSF
C nastavuji hodnoty IPU i konkretni parametry) a teprve pred volanim FEFRON je
C transformovan do V(*), ktery pak FEFRON nahradi vyslednym resenim.
C Pri transformaci Vo->V se provadi tyto konverze:
C   1. Je-li -10<=IPU<0  vola se transformacni funkce Vo.f(IPU,x,y,t,T)->V
C   2. Je-li   0<=IPU<30 transformace bodovych zatizeni, Vo.f(IPU,x,y,t,T)->V
C   3. Je-li  30<=IPU<50 odpovidajici prvky vektoru V(*) se nuluji 0->V a do vektoru
C                   Vo se priradi hodnoty pocitane funkci f(IPU,x,y,t,T)->Vo.
C
C    V programu COSMOS.FOR jsou zavedeny tyto dalsi konvence vyznamu prvku
C vektoru IPU(*), ktere maji vyznam pro cinnost procedury FEL:
C    IPU<0      silne okrajove podminky se v definici FEL neuplatni
C    0<=IPU<10  pouze bodove zdroje, ani ty se v definici FEL neuplatni.
C Slabe okrajove podminky kombinovane s bodovymi zdroji, parametry
C okrajovych podminek (napr. hustota toku, alfa, Te) musi byt konstantni:
C    10<=IPU<20  bodove zdroje a okrajove podminky druheho druhu
C    20<=IPU<30  bodove zdroje a okrajove podminky tretiho druhu
C Slabe okrajove podminky bez bodovych zdroju, ktere ale mohou vyuzivat
C promenne uzlove parametry Vo k definici menicich se okrajovych podminek,
C napr. spojite se meniciho zatizeni podel hranice oblasti:
C    30<=IPU<40  okrajove podminky druheho druhu bez bodovych zdroju
C    40<=IPU<50  okrajove podminky tretiho druhu bez bodovych zdroju.
C Vyse zminovane transformacni funkce f(IPU,x,y,t,T) definovane pro
C hodnoty indexu -10<=IPU<50 (az 60 ruznych funkci) mohou byt tri ruznych typu
C     - kontanta f=1  identicka transformace (tj. default)
C     - analyticky zadana funkce (muze to byt i program interpretovany TINQ)
C     - linearni interpolace v tabulce (stejne jako COSMOS/M)
C
C PROCEDURY, RESPEKTUJICI TENTO ZPUSOB DEFINICE ELEMENTU JSOU
C FEFRIN, FEFRON
C FEMSF   (GENEROVANI TROJUHELNIKU/CTYRUHELNIKU NA PLOSE)
C GMFCQND (KRESLENI VRSTEVNIC)
C FEMERGE (SPOJOVANI UZLU SE STEJNYMI SOURADNICEMI)
C
C--------------------------------------------------------------------------
C      SUBROUTINE FEFRON(ICONTR,IPU,LPU,MPU,X,IUE,LUE,MUE,NE,EPS,IDVP,FEL)
C VSTUPNI PARAMETRY
C ICONTR    - ICONTR=0 standardni cinnost, tj. sestaveni a reseni soustavy rovnic
C             ICONTR=1 neprovadi se ani sestaveni ani reseni, ale elementy
C                      se zpracovavaji standardne a procedure FEL se predavaji
C                      hodnoty X. Pouzije-li se volani FEFRON v tomto rezimu az
C                      po vyreseni soustavy rovnic, muze procedura FEL zpracovavat
C                      vysledky tohoto reseni.
C VSTUPNI PARAMETRY POPISUJICI UZLOVE BODY
C IPU(NEQ)  - status parametru X(IEQ), IEQ=1,2,...,NEQ.
C             IPU=0  NEZNAMA, NEFIXOVA OKRAJOVOU PODMINKOU, ZADNE ZATIZENI.
C             IPU>0  NEZNAMA, NA VSTUPU DO FEFRON REPREZENTUJE PRISPEVEK K
C                    VEKTORU PRAVE STRANY (PREDAVA SE PROCEDURE FEL)
C             IPU<0  OKRAJOVA PODMINKA PRVNIHO DRUHU (ZADAVANA HODNOTA X())
C             ABS(IPU)>1E6 PASIVOVANY PARAMETR (PRI RESENI SE NEBERE V UVAHU)
C LPU(NU)   - INDEX PRVNIHO PARAMETRU UZLU VE VEKTORECH IPU(NEQ),X(NEQ) MINUS 1
C MPU(NU)   - POCET UZLOVYCH PARAMETRU UZLU NU
C X(NEQ)    - VEKTOR UZLOVYCH PARAMETRU X(IEQ), IEQ=1,2,...,NEQ.
C               PRIKLAD: K-TY PARAMETR UZLU IU JE X(LPU(IU)+K).
C VSTUPNI PARAMETRY POPISUJICI ELEMENTY
C IUE(*)    - VEKTOR KONEKTIVITY (IUE<0 ZNAMENA POSLEDNI VYSKYT UZLU)
C LUE(NE+1) - INDEX IUE(*) PRVNIHO UZLU ELEMENTU MINUS 1.
C MUE(NE)   - IABS(MUE(IE))=POCET UZLU ELEMENTU IE (rovnez LUE(IE+1)-IE(IE))
C NE        - POCET ELEMENTU
C EPS       - MINIMALNI PRIPUSTNA VELIKOST PIVOTACNIHO PRVKU
C IDVP      - CISLO ZARIZENI (KDYZ IDVP=0, ZAPIS DO PAMETI)
C FUNKCE POPISUJICI ALGORITMUS ELEMENTU
C     CALL FEL(ICONTR,IE,L,NUE,AL,BL)
C ICONTR    - PREDAVANA HODNOTA PARAMETRU FEFRON
C IE        - INDEX ELEMENTU
C L         - JE ROZMER LOKALNI MATICE AL(L,L)
C AL(L,L)   - LOKALNI MATICE ELEMENTU
C BL(L)     - LOKALNI VEKTOR PRAVE STRANY
C Na vystupu BL predava procedura FEL objemovy zdroj. Pokud je nulovy,
C musi procedura FEL tento vektor vynulovat.
C--------------------------------------------------------------------------
C
C GRAFIKA
C---------
C DEFINICI TRANSFORMACI ZAJISTUJE SKUPINA PROCEDUR GMFSTx .
C NEKTERE Z TECHTO PROCEDUR JSOU POPLATNE STARSI KNIHOVNE - STACILA BY
C TOTIZ JEN JEDINA (NOVA) INICIALIZACNI PROCEDURA
C     GMFSTW
C URCENA PRO PRACI V REZIMU QUICK-WIN (GRAFY V OKNECH, OTEVRENYCH PROCEDUROU
C WOPEN). V PROCEDURE GMFSTW SE DEFINUJE KORESPONDENCE MEZI DRIVE VYTVORENYM
C WINDOWSOVSKYM OKNEM (IWIN) A CISLEM GRAFU (IND). POD POJMEM GRAF SE
C ROZUMI DEFINICE ROZSAHU SOURADNIC VNEJSIHO A VNITRNIHO OBDELNIKOVEHO OKENKA
C (VNITRNI OKENKO JE VYMEZENO SOURADNICOVYMI OSAMI, PLOCHA MEZI VNITRNIM A
C VNEJSIM OKENKEM JE VYHRAZENA PRO POPIS SOURADNYCH OS), DATA GRAFU URCUJI
C TYP A VELIKOSTI FONTU, POPISY OS. PARAMETR -ITYPE- URCUJE POMER
C VELIKOSTI VNITRNIHO A VNEJSIHO OKENKA GRAFU A ZDA SE ZAOKROUHLUJI ROZSAHY OS:
C     ITYPE=0   FIXOVANY ROZSAH SOURADNIC, VHODNE PRO VYKRESLOVANI VRSTEVNIC
C     ITYPE=1   ZAOKROUHLOVANI ROZSAHU SOURADNIC, VHODNE PRO ARITMETICKE OSY
C     ITYPE=2   ZAOKROUHLOVANI ROZSAHU SOURADNIC, VHODNE PRO LOG-LOG OSY.
C V JEDNOM OKNE (CHILD WINDOW) LZE ZOBRAZIT I NEKOLIK GRAFU.
C ODVOLAVAME-LI SE POZDEJI NA GRAF CISLO -IND- JE DEFINOVANO I TO,
C VE KTEREM OKNE (CHILD WINDOW) SE TENTO GRAF OBJEVI.
C
C I STARSI PROCEDURY GMFST,GMFSTE,GMFSTF SE OVSEM MOHOU V REZIMU QUICK WIN
C POUZIVAT (PROCEDURU GMFST SPISE JEN Z DUVODU KOMPATIBILITY).
C     GMFSTF - NA ROZDIL OD OSTATNICH NEKRESLI OSY (VNITRNI OKNO=VNEJSI OKNO).
C              TATO PROCEDURA NENI URCENA PRO GRAFY, ALE JEN PRO VOLNE KRESLENI
C              VE VYMEZENE PLOSE.
C     GMFSTE - ODPOVIDA PROCEDURE GMFSTW. VARIANTY ITYPE=0,1,2 JSOU ZAKODOVANY
C              PONEKUD NESIKOVNE DO PARAMETRU IND (INDEX GRAFU).
C              ZAPORNA HODNOTA INDEXU GRAFU IND ODPOVIDA ITYPE=2 (LOG),
C              KLADNE HODNOTY 1,...6,   ITYPE=1 (ARITMETICKE OSY)
C              KLADNE HODNOTY 11,...16, ITYPE=0 (VRSTEVNICE).
C
      END
