      SUBROUTINE FDFT(NOD,X,Y,RL1,RL2,F,FX,FY,S)
C
C ISOPARAMETRICKY TROJUHELNIKOVY PRVEK (XI,YI) SE 3,4,6,7 UZLOVYMI BODY.
C F1,...F(6)  =1 V UZLECH
C
C VSTUPY:
C X(),Y()       SOURADNICE UZLU (1,2,3 -VRCHOLY, 4,5,6-STREDY STRAN)
C RL1,RL2=L1,L2 PLOSNE SOURADNICE LIBOVOLNEHO BODU (0,1)
C
C NOD=-3   3 linearni bazove funkce odpovidajici uzlum ve stredech stran 
C NOD=3    3 linearni bazove funkce odpovidajici vrcholum
C NOD=4    3 linearni bazove funkce vrcholu, kubicka bublina ctvrteho uzlu
C NOD=6    6 kvadratickych bazovych funkci (vrcholy a stredy stran)
C NOD=7    6 kvadratickych bazovych funkci, kubicka bublina sedmeho uzlu 
C
C VYSTUP:
C F,FX,FY HODNOTY A PRVNI DERIVACE TVAROVYCH FUNKCI V BODE L1,L2
C S=PLOCHA TROJUHELNIKU (=JAKOBIAN/2)
C
      DIMENSION Y(*),X(*),F(*),FY(*),FX(*)
      REAL L(3),LY(3),LX(3)
      L(1)=RL1
      L(2)=RL2
      L(3)=1.-RL1-RL2
      SELECT CASE(NOD)
      CASE(3)
C 3-UZLY (LINEARNI TVAROVE FUNKCE); TESTED O.K. BY BTEST.FOR
C AREA
         DET=(Y(1)*(X(2)-X(3))+Y(2)*(X(3)-X(1))+Y(3)*(X(1)-X(2)))
         IF(DET.EQ.0.)RETURN
         S=ABS(DET*.5)
C DERIVATIVES OF AREA COORDINATES
         DO I=1,3
           I1=MOD(I,3)+1
           I2=MOD(I1,3)+1
           FY(I)=(X(I1)-X(I2))/DET
           FX(I)=(Y(I2)-Y(I1))/DET
           F(I)=L(I)
         ENDDO
      CASE(-3)
C 3-UZLY ve stredech stran X(i),Y(i) jsou ale souradnice vrcholu
         DET=(Y(1)*(X(2)-X(3))+Y(2)*(X(3)-X(1))+Y(3)*(X(1)-X(2)))
         IF(DET.EQ.0.)RETURN
         S=ABS(DET*.5)
         DO I=1,3
           I1=MOD(I,3)+1
           I2=MOD(I1,3)+1
           LY(I)=(X(I1)-X(I2))/DET
           LX(I)=(Y(I2)-Y(I1))/DET
           F(I)=1.-2.*L(I)
           FX(I)=-2.*LX(I)
           FY(I)=-2.*LY(I)
         ENDDO
      CASE(4)
C 3-UZLY (LINEARNI TVAROVE FUNKCE); 4-UZEL BUBLINOVA FUNKCE (KUBICKA)
         DET=(Y(1)*(X(2)-X(3))+Y(2)*(X(3)-X(1))+Y(3)*(X(1)-X(2)))
         IF(DET.EQ.0.)RETURN
         S=ABS(DET*.5)
         DO I=1,3
           I1=MOD(I,3)+1
           I2=MOD(I1,3)+1
           FY(I)=(X(I1)-X(I2))/DET
           FX(I)=(Y(I2)-Y(I1))/DET
           F(I)=L(I)
         ENDDO
         F(4)=27.*L(1)*L(2)*L(3)
         FX(4)=27.*(FX(1)*L(2)*L(3)+FX(2)*L(1)*L(3)+FX(3)*L(1)*L(2))
         FY(4)=27.*(FY(1)*L(2)*L(3)+FY(2)*L(1)*L(3)+FY(3)*L(1)*L(2))
      CASE(6)
C 6-UZLU (KVADRATICKE ISOPARAMETRICKE FUNKCE)
         SX1=(4*L(1)-1)*X(1)-(4*L(3)-1)*X(3)+
     /        4*(L(2)*X(4)+L(3)*X(6)-L(2)*X(5)-L(1)*X(6))
         SX2=(4*L(2)-1)*X(2)-(4*L(3)-1)*X(3)+
     /        4*(L(1)*X(4)+L(3)*X(5)-L(2)*X(5)-L(1)*X(6))
         SY1=(4*L(1)-1)*Y(1)-(4*L(3)-1)*Y(3)+
     /        4*(L(2)*Y(4)+L(3)*Y(6)-L(2)*Y(5)-L(1)*Y(6))
         SY2=(4*L(2)-1)*Y(2)-(4*L(3)-1)*Y(3)+
     /        4*(L(1)*Y(4)+L(3)*Y(5)-L(2)*Y(5)-L(1)*Y(6))
         DET=SX1*SY2-SX2*SY1
         IF(DET.EQ.0.)RETURN
         S=ABS(DET*.5)
         LX(1)=SY2/DET
         LX(2)=-SY1/DET
         LX(3)=-LX(1)-LX(2)
         LY(1)=SX2/DET
         LY(2)=-SX1/DET
         LY(3)=-LY(1)-LY(2)
         DO I=1,3
           I1=MOD(I,3)+1
           F(I)=L(I)*(2*L(I)-1)
           F(I+3)=4*L(I)*L(I1)
           FX(I)=LX(I)*(4*L(I)-1)
           FX(I+3)=4*(L(I1)*LX(I)+L(I)*LX(I1))
           FY(I)=-LY(I)*(4*L(I)-1)
           FY(I+3)=-4*(L(I1)*LY(I)+L(I)*LY(I1))
         ENDDO
      CASE(7)
C 6-UZLU (KVADRATICKE ISOPARAMETRICKE FUNKCE) 7-UZEL (KUBICKA BUBLINA)
C BUBLINA
         DET=(Y(1)*(X(2)-X(3))+Y(2)*(X(3)-X(1))+Y(3)*(X(1)-X(2)))
         IF(DET.EQ.0.)RETURN
         DO I=1,3
           I1=MOD(I,3)+1
           I2=MOD(I1,3)+1
           LY(I)=(X(I1)-X(I2))/DET
           LX(I)=(Y(I2)-Y(I1))/DET
         ENDDO
         F(7)=27.*L(1)*L(2)*L(3)
         FX(7)=27.*(LX(1)*L(2)*L(3)+LX(2)*L(1)*L(3)+LX(3)*L(1)*L(2))
         FY(7)=27.*(LY(1)*L(2)*L(3)+LY(2)*L(1)*L(3)+LY(3)*L(1)*L(2))
C KVADRATICKE FUNKCE
         SX1=(4*L(1)-1)*X(1)-(4*L(3)-1)*X(3)+
     /        4*(L(2)*X(4)+L(3)*X(6)-L(2)*X(5)-L(1)*X(6))
         SX2=(4*L(2)-1)*X(2)-(4*L(3)-1)*X(3)+
     /        4*(L(1)*X(4)+L(3)*X(5)-L(2)*X(5)-L(1)*X(6))
         SY1=(4*L(1)-1)*Y(1)-(4*L(3)-1)*Y(3)+
     /        4*(L(2)*Y(4)+L(3)*Y(6)-L(2)*Y(5)-L(1)*Y(6))
         SY2=(4*L(2)-1)*Y(2)-(4*L(3)-1)*Y(3)+
     /        4*(L(1)*Y(4)+L(3)*Y(5)-L(2)*Y(5)-L(1)*Y(6))
         DET=SX1*SY2-SX2*SY1
         IF(DET.EQ.0.)RETURN
         S=ABS(DET*.5)
         LX(1)=SY2/DET
         LX(2)=-SY1/DET
         LX(3)=-LX(1)-LX(2)
         LY(1)=SX2/DET
         LY(2)=-SX1/DET
         LY(3)=-LY(1)-LY(2)
         DO I=1,3
           I1=MOD(I,3)+1
           F(I)=L(I)*(2*L(I)-1)
           F(I+3)=4*L(I)*L(I1)
           FX(I)=LX(I)*(4*L(I)-1)
           FX(I+3)=4*(L(I1)*LX(I)+L(I)*LX(I1))
           FY(I)=-LY(I)*(4*L(I)-1)
           FY(I+3)=-4*(L(I1)*LY(I)+L(I)*LY(I1))
         ENDDO
      END SELECT
      END
 
