      SUBROUTINE FFFRON(ICTR,LPU,NP,IPU,X,NDIM,
     /                  IUE,LUE,MUE,NE,EPS,NFLIM,IDVP,FEL)
C
C FRONTALNI METODA - UPRAVENA VARIANTA PROCEDURY FEFRON, UVAZUJICI MATICI
C                    NEZNAMYCH A STATUSU PARAMETRU (TJ. FIXNI POCET
C                    UZLOVYCH PARAMETRU).
C
C VSTUPNI PARAMETRY
C ICTR      - ICTR=0 sestaveni a reseni soustavy rovnic
C             ICTR=1 neprovadi se ani sestaveni ani reseni, ale elementy
C                    se zpracovavaji standardne a procedure FEL se predavaji
C                    hodnoty X. Pouzije-li se volani FFFRON v tomto rezimu az
C                    po vyreseni soustavy, muze procedura FEL zpracovavat
C                    vysledky tohoto reseni.
C LPU(NP)   - INDEXY RESENYCH DOF (RADKU MATICE NEZNAMYCH X(NDIM,ND))
C NP        - POCET UVAZOVANYCH DOF UZLOVYCH BODU (N-Parametru uzlu)
C
C VSTUPNI PARAMETRY POPISUJICI UZLOVE BODY JSOU DEKLAROVANY JAKO VEKTORY IPU,X,
C MAJICICH ALE CHARAKTER DVOUROZMERNE MATICE IPU(ND,IP) A X(ND,IP), NAPR.
C TEDU PRVNI SLOUPEC TEPLOTY, DRUHY SLOUPEC POSUVY UX,... NDIM-JE MAXIMALNI
C POCET UZLU A JE TO ZAROVEN PARAMETR MAPOVACI FUNKCE.
C
C IPU(*)    - STATUS PARAMETRU UZLU ND.
C             IPU=0  NEZNAMA, NEFIXOVA OKRAJOVOU PODMINKOU, ZADNE ZATIZENI.
C             IPU>0  NEZNAMA, NA VSTUPU DO FFFRON REPREZENTUJE PRISPEVEK K
C                    VEKTORU PRAVE STRANY (PREDAVA SE PROCEDURE FEL)
C             IPU<0  OKRAJOVA PODMINKA PRVNIHO DRUHU (ZADAVANA HODNOTA X())
C             ABS(IPU)>1E6 PASIVOVANY PARAMETR (PRI RESENI SE NEBERE V UVAHU)
C X(*)      - MATICE UZLOVYCH PARAMETRU X(I,ND), JE I-TY PARAMETRU UZLU ND.
C VSTUPNI PARAMETRY POPISUJICI ELEMENTY
C IUE(*)    - VEKTOR KONEKTIVITY (IUE<0 ZNAMENA POSLEDNI VYSKYT UZLU)
C LUE(NE+1) - INDEX IUE(*) PRVNIHO UZLU ELEMENTU MINUS 1.
C MUE(NE)   - IABS(MUE(IE))=POCET UZLU ELEMENTU IE (rovnez LUE(IE+1)-IE(IE))
C NE        - POCET ELEMENTU
C EPS       - MINIMALNI PRIPUSTNA VELIKOST PIVOTACNIHO PRVKU
C NFLIM     - URCUJE MINIMALNI POCET DOKOMPLETOVANYCH ROVNIC S NENULOVYMI PIVOTY,
C             KDY JIZ ZACNE ELIMINACE
C IDVP      - CISLO ZARIZENI (KDYZ IDVP=0, ZAPIS DO PAMETI)
C FUNKCE POPISUJICI ALGORITMUS ELEMENTU
C     CALL FEL(ICTR,IE,L,NUE,AL,BL)
C ICTR      - PREDAVANA HODNOTA PARAMETRU FFFRON
C IE        - INDEX ELEMENTU
C L         - JE ROZMER LOKALNI MATICE AL(L,L)
C AL(L,L)   - LOKALNI MATICE ELEMENTU
C BL(L)     - LOKALNI VEKTOR PRAVE STRANY
C Na vystupu BL predava procedura FEL objemovy zdroj. Pokud je nulovy,
C musi procedura FEL tento vektor vynulovat.
C
C POMOCNE PARAMETRY
C AG(NF,NF)   - GLOBALNI MATICE SOUSTAVY
C AL(NL*NL)   - LOKALNI MATICE SOUSTAVY
C BL  (NL)    - LOKALNI VEKTOR PRAVE STRANY (OBJEMOVY ZDROJ)
C LG  (NL)    - LG(I) RADEK MATICE AG ODPOVIDAJICI I-TEMU RADKU MATICE AL
C LLN (NL)    - LLN(I)INDEX UZLU ODPOVIDAJICI I-TEMU RADKU MATICE AL.
C               LLN(I)<0 ZNAMENA, ZE RADEK ODPOVIDA OKRAJOVE PODMINCE 1.DRUHU
C LLX (NL)    - LLX>0 INDEX X()
C               LLX<0 INDEX X() PRO DOKOMPLETOVANOU ROVNICI
C LGX (NF)    - TOTEZ CO LLX(), TJ. INDEX X()
C PN  (NF)    - POMOCNY VEKTOR PIVOTU
C
C Pouzivane procedury: FEFWR$(sekvencni WRITE), FEFRE$(READ),
C                      FEFOP$(OPEN), FEFBC$ (simulace BACKSPACE).
C
      PARAMETER (NF=200,NL=24,IMA=1000000)
      DIMENSION LPU(*),IPU(*),X(*),IUE(*),LUE(*),MUE(*)
      DIMENSION AG(NF,NF),AL(NL*NL),BL(NL),PN(NF),LGX(NF),LLX(NL),
     /          LLN(NL),LG(NL),LW(2)
      COMMON /FEFRON$/IDV,IA(IMA),IRC
      DIMENSION IPN(NF)
      EQUIVALENCE (IPN,PN)
      EXTERNAL FEL
      IDV=IDVP
C
C Otevreni IDV pro zapis
C
      DO I=1,NF
      DO J=1,NF
        AG(I,J)=0.
      ENDDO
      ENDDO
      CALL FEFOP$
      NREC=0
      NFA=0
      J=1
C
C <-------IE-cyklus--------->
C
      ELOOP: DO IE=1,NE
        LE0=LUE(IE)
C FFFRON uvazuje jen absolutni hodnotu MUE (pocet uzlu); zaporna hodnota
C indikuje to, ze posledni uzel je v tezisti elementu, coz ma vyznam jen pro
C grafiku a ne pro reseni.
        NUE=IABS(MUE(IE))
        L=0
        DO I=1,NUE
           INDU =IUE(LE0+I)
           INDUA=IABS(INDU)
C
C INDU-globalni index I-teho uzlu v elementu IE.
C NP  -pocet parametru uzlu
C
C Nasledujici blok stanovi korespondenci mezi radky lokalni matice
C a indexy X().
C ----------------------------------------------------------------
           DO IP=1,NP
C MATICE IPU(),X() JSOU MAPOVANE DLE NDIM (POCET RADKU), LPU(IP) JE
C INDEX SLOUPCE A INDUA INDEX RADKU (CISLO UZLU)
             IX=NDIM*(LPU(IP)-1)+INDUA
             ISTA=IPU(IX)
C
C testovani typu uzloveho parametru (pasivovany, o.p., neznama)
C
             IF(IABS(ISTA).LT.1000000)THEN
                L=L+1
C Ulozeni puvodni hodnoty X() do lokalniho vektoru prave strany BL
C Tato hodnota se muze uplatnit pri vyhodnocovani vysledku
C (napr.vypoctu gradientu)
                BL(L)=X(IX)
C Pro dokompletovane parametry bude LLX<0
                LLX(L)=IX
                IF(INDU.LT.0) LLX(L)=-IX
C Pro fixovane parametry (o.p.) bude LLN(L)<0
                LLN(L)=INDUA
                IF(ISTA.LT.0) LLN(L)=-INDUA
             ENDIF
          ENDDO
        ENDDO
C
C ---KONEC CYKLU PRES UZLU ELEMENTU IE
C
C L  -je v tomto okamziku rovno poctu radku lokalni matice, ktere je
C     treba zahrnout do sestavovane matice AG (L<=NL)
C NFA-je aktualni rozmer globalni matice AG (na zacatku sestaveni 0)
C
C VOLANI UZIVATELSKE PROCEDURY, L JE ROZMER LOKALNI MATICE
C Na vystupu BL predava procedura FEL objemovy zdroj. Pokud je nulovy,
C musi procedura FEL tento vektor vynulovat.
C
        CALL FEL(ICTR,IE,L,NUE,AL,BL)
        IF(ICTR.NE.0) CYCLE ELOOP
C
C Nasledujici blok stanovi korespondenci mezi radky lokalni a globalni
C matice (vektor LG) a korespondenci mezi globalni matici a vektorem X().
C ----------------------------------------------------------------
        DO I=1,L
          IF(LLN(I).GT.0)THEN
            IF(NFA.LE.0)THEN
               NFA=1
               J=1
            ELSE
               DO J=1,NFA
                  IF(IABS(LGX(J)).EQ.IABS(LLX(I))) GOTO 35
               ENDDO
               NFA=NFA+1
               J=NFA
               IF(NFA.GT.NF)THEN
                   WRITE(*,*)' PRILIS MALA MATICE AG'
                   RETURN
               ENDIF
            ENDIF
35          LG(I)=J
            LGX(J)=LLX(I)
          ENDIF
        ENDDO
C
C PRICTENI LOKALNI MATICE A(IJ) KE GLOBALNI MATICI AG(NFA,NFA)
C PRICTENI LOKALNIHO VEKTORU B(I) KE GLOBALNIMU VEKTORU ZATIZENI X()
C       Pri sestaveni jsou pouzity vektory LG a LLX vysvetlujici kores-
C pondenci mezi radky lokalni a globalni matice (AL,AG) a vektorem nez-
C namych X.
C LG - indexy radku globalni matice AG.
C LLN- index uzlu (vyuzito jen pri aktivaci externi funkce FBC).
C LLX- indexy neznamych X. LLX<0 je informace o tom, ze rovnice je uplna.
C
        DO I=1,L
          I0=LG(I)
          IF(LLN(I).GT.0)THEN
C LLN>0 ZNAMENA, ZE UVAZUJEME JEN ROVNICE, NEFIXOVANE OKRAJOVOU PODMINKOU
            IR=IABS(LLX(I))
            X(IR)=X(IR)+BL(I)
            IJ=I
            DO J=1,L
              J0=LG(J)
              IF(LLN(J).GT.0)THEN
C Parametr neni blokovan okrajovou podminkou - PRICTENI
                   AG(I0,J0)=AG(I0,J0)+AL(IJ)
              ELSE
C Parametry zadavane jako okrajove podminky prevedeme na pravou stranu
                   IX=IABS(LLX(J))
                   X(IR)=X(IR)-AL(IJ)*X(IX)
              ENDIF
              IJ=IJ+L
            ENDDO
          ENDIF
        ENDDO
C
C Pocet nenulovych a dokompletovanych pivotacnich prvku
C
        NPIV=0
        DO I=1,NFA
          IF(LGX(I).LT.0.AND.AG(I,I).NE.0.)NPIV=NPIV+1
        ENDDO
C
C Test, zda uz je zadouci provest eliminaci a vlastni eliminace:
C
      ELIM: DO WHILE((NFA.GT.NF-NL .AND.IE.LT.NE) .OR.
     /               (NPIV.GT.NFLIM.AND.IE.LT.NE) .OR.
     /               (NFA.GT.0     .AND.IE.EQ.NE))
          NPIV=NPIV-1
          PIVM=0.
          DO I=1,NFA
            IF(LGX(I).LT.0)THEN
C
C Vyber hlavniho prvku z diagonaly dokompletovaneho radku
C (LGX<0 DOKOMPLETOVANA ROVNICE)
C
              PIV=AG(I,I)
              IF(ABS(PIV).GT.ABS(PIVM)) THEN
                PIVM=PIV
                LPIV=I
              ENDIF
            ENDIF
          ENDDO
C Bude treba provest eliminaci jednoho radku globalni matice AG.
C Z dokompletovanych rovnic (viz. zaporne prvky vektoru LGX) je vybran
C pivotacni prvek (nejvetsi prvek AG na diagonale)
          IF(ABS(PIVM).LT.EPS)THEN
            WRITE(*,*)' PIVOT',PIVM,' MENSI NEZ',EPS
            RETURN
          ENDIF
          KP=IABS(LGX(LPIV))
          DO I=1,NFA
             PN(I)=AG(LPIV,I)/PIVM
          ENDDO
          X(KP)=X(KP)/PIVM
          XK=X(KP)
          DO I=1,NFA
C UPRAVA VSECH RADKU S VYJIMKOU PIVOTACNIHO
            IF(I.NE.LPIV)THEN
                AA=AG(I,LPIV)
                K=IABS(LGX(I))
                X(K)=X(K)-AA*XK
C RADKY NAD PIVOTACNIM PRESOUVAME O JEDNICKU DOLU
                I0=I
                IF(I.GT.LPIV)I0=I0-1
                DO J=1,NFA
                   IF(J.NE.LPIV)THEN
                      J0=J
                      IF(J.GT.LPIV)J0=J0-1
                      AG(I0,J0)=AG(I,J)-AA*PN(J)
                   ENDIF
                ENDDO
            ENDIF
          ENDDO
C
C Neformatovany sekvencni zapis techo velicin:
C NFA,LPIV, LGX1,...,LGX(NFA), PN1,...,PN(NFA)
C
          LW(1)=NFA
          LW(2)=LPIV
          NREC=NREC+1
          CALL FEFWR$(2,LW)
          CALL FEFWR$(NFA,LGX)
          CALL FEFWR$(NFA,IPN)
C VYNULOVANI POSLEDNIHO RADKU A SLOUPCE AG
          DO I=1,NFA
            AG(I,NFA)=0.
            AG(NFA,I)=0.
          ENDDO
          NFA=NFA-1
          DO I=LPIV,NFA
            LGX(I)=LGX(I+1)
          ENDDO
        ENDDO ELIM
      ENDDO ELOOP
C
      IF(ICTR.NE.0)RETURN
C
C ZPETNA SUBSTITUCE
C
      DO IR=1,NREC
C
C BACKSPACE 3*
C
         CALL FEFBC$(3)
         CALL FEFRE$(2,LW)
         NFA=LW(1)
         LPIV=LW(2)
         CALL FEFRE$(NFA,LGX)
         CALL FEFRE$(NFA,IPN)
         KP=IABS(LGX(LPIV))
         TS=0.
         PN(LPIV)=0.
         DO I=1,NFA
           TS=TS-PN(I)*X(IABS(LGX(I)))
         ENDDO
         X(KP)=X(KP)+TS
         CALL FEFBC$(3)
      ENDDO
C
C KONEC RESENI
C
      CONTAINS
C
C POMOCNE PROCEDURY PRO SEKVENCNI ZAPIS A CTENI
C----------------------------------------------
      SUBROUTINE FEFOP$
C
C OPEN souboru IDV. Pri IDV=0 je simulovano otevreni virtualniho souboru.
C
      PARAMETER (IMA=1000000)
      COMMON /FEFRON$/IDV,IA(IMA),IRC
      IF(IDV.GT.0)THEN
        REWIND IDV
      ELSE
        IRC=1
      ENDIF
      END SUBROUTINE FEFOP$
 
      SUBROUTINE FEFRE$(N,IV)
C
C Procedura FEFRE$ precte N-slov vektoru IV() ze zarizeni IDV.
C
      PARAMETER (IMA=1000000)
      COMMON /FEFRON$/IDV,IA(IMA),IRC
      DIMENSION IV(*)
      IF(IDV.GT.0) THEN
C CTENI ZE SOUBORU
         READ(IDV)(IV(I),I=1,N)
         RETURN
      ELSE
C CTENI Z COMMON//
         IR1=IRC+1
         IR2=IRC+N
         IF(IR2.GT.IMA)THEN
             WRITE(*,*)' CTENI Z PRILIS KRATKE ZONY IA//'
             RETURN
         ENDIF
         J=1
         DO I=IR1,IR2
           IV(J)=IA(I)
           J=J+1
         ENDDO
         IA(I)=N+1
         IRC=IR2+1
      ENDIF
      END SUBROUTINE FEFRE$
 
      SUBROUTINE FEFWR$(N,IV)
C
C Procedura FEFWR$ zapise N-slov vektoru IV() na zariceni IDV.
C
      PARAMETER (IMA=1000000)
      COMMON /FEFRON$/IDV,IA(IMA),IRC
      DIMENSION IV(*)
      IF(IDV.GT.0)THEN
C ZAPIS DO SOUBORU
         WRITE(IDV)(IV(I),I=1,N)
      ELSE
C ZAPIS DO COMMON//
         IR1=IRC+1
         IR2=IRC+N
         IF(IR2.GE.IMA)THEN
             WRITE(*,*)' PRILIS MALA ZONA COMMON//',IMA
             RETURN
         ENDIF
         J=1
         DO I=IR1,IR2
           IA(I)=IV(J)
           J=J+1
         ENDDO
         IA(I)=N+1
         IRC=IR2+1
      ENDIF
      END SUBROUTINE FEFWR$
 
      SUBROUTINE FEFBC$(N)
C
C BACKSPACE IDV celkem N krat. Pri IDV je tato operace simulovana ve
C vnitrni pameti.
C
      PARAMETER (IMA=1000000)
      COMMON /FEFRON$/IDV,IA(IMA),IRC
      IF(IDV.GT.0)THEN
         DO I=1,N
            BACKSPACE IDV
         ENDDO
         RETURN
      ELSEIF(IRC.LE.IMA)THEN
         DO I=1,N
           NN=IA(IRC)
           IRC=IRC-NN
         ENDDO
      ENDIF
      END SUBROUTINE FEFBC$
 
      END
 
 
