      SUBROUTINE FFMSF(X8,Y8,N8,NIX,NIY,AX,AY,NUID,
     /                 X,Y,KINDU,NDOLD,NDNEW,IUE,LUE,MUE,NEOLD,NENEW)
C
C GENEROVANI CTYRUHELNIKOVYCH NEBO TROJUHELNIKOVYCH ELEMENTU
C V ZADANEM CTYRUHELNIKU DEFINOVANEM 4 NEBO 8 BODY (PT).
C
C GENEROVANE ELEMENTY MOHOU MIT RUZNE POCTY UZLU A RUZNY POCET PARAMETRU
C V UZLOVYCH BODECH. JE MOZNE DEFINOVAT TYTO TYPY ELEMENTU:
C  NUID=3   TROJUHELNIK SE 3 UZLY
C  NUID=4   CTYRUHELNIK SE 4 UZLY
C  NUID=6   TROJUHELNIK SE 6 UZLY (1,2,3 VRCHOLY, 4,5,6 STREDY STRAN)
C  NUID=8   CTYRUHELNIK S  8 UZLY (1,2,3,4 VRCHOLY, 5,6,7,8 STREDY STRAN)
C  NUID=-4  TROJUHELNIK SE 4 UZLY (UZEL 4 TEZISTE)
C  NUID=-5  CTYRUHELNIK S  5 UZLY (UZEL 5 TEZISTE)
C  NUID=-7  TROJUHELNIK SE 7 UZLY (UZEL 7 TEZISTE)
C  NUID=-9  CTYRUHELNIK S  9 UZLY (UZEL 9 TEZISTE)
C
C VSTUPNI PARAMETRY:
C X8(*),Y8(*) - SOURADNICE DEFINICNICH BODU CTYRUHELNIKOVE PLOCHY,
C               PRVNI 4 BODY JSOU VRCHOLY A DALSI CTVERICE UPROSTRED STRAN
C N8          - POCET DEFINICNICH BODU (PRIPUSTNE HODNOTY 4 NEBO 8)
C NIX,NIY     - POCET INTERVALU DELICICH STRANY 1 A 2
C AX,AY       - ZHUSTENI ELEMENTU (FEMDIV, zaporne hodnoty symetricke zhusteni)
C NUID        - ABS(NUID)=POCET UZLU ELEMENTU (NUID<0 UZEL V TEZISTI)
C NDOLD       - NOVE UZLY BUDOU GENEROVANY INDEXEM NDOLD+1 POCINAJE
C               NOVE ODKAZY NA VEKTOR PARAMETRU IPU INDEXEM NDOLD+1 POCINAJE
C NEOLD       - NOVE ELEMENTY BUDOU VYTVARENY INDEXEM NEOLD+1 POCINAJE
C
C VYSTUPNI PARAMETRY:
C NDNEW       - INDEX POSLEDNIHO GENEROVANEHO UZLU
C NENEW       - INDEX POSLEDNIHO GENEROVANEHO ELEMENTU
C X(N),Y(N)   - SOURADNICE UZLOVYCH BODU
C KINDU(N)    - TYP UZLU
C               KINDU=1 (VRCHOLY), KINDU=2 (NA STRANACH), KINDU=3 (V TEZISTI)
C               KINDU=0 ZRUSENY UZEL.
C IUE(*)      - VEKTOR KONEKTIVITY ELEMENTU
C LUE(*)      - LUE(IE)+I INDEX I-TEHO UZLU ELEMENTU IE.
C               NOVE SE GENERUJI PRVKY LUE(NEOLD+2),...,LUE(NENEW+1)
C MUE(*)     -  NUID (VYUZIVANO PRO GRAFIKU, MUE<0 POSLEDNI UZEL V TEZISTI)
C
      PARAMETER (MAXU=200,MAXEL=10000,MAXND=10000)
      DIMENSION X8(*),Y8(*),X(*),Y(*),IUE(*),LUE(*),MUE(*),
     /          KINDU(*)
      DIMENSION RX(8),RY(8),XA(MAXU),YA(MAXU)
      LOGICAL XODD,YODD
      DATA RX /-1.,1.,1.,-1.,0.,1.,0.,-1./
      DATA RY /-1.,-1.,1.,1.,-1.,0.,1.,0./
C NU=POCET UZLU ELEMENTU, NUA-POCET UZLU PRO GRAFIKU
      NU=IABS(NUID)
      IF(NUID.GT.0)THEN
         NUA=NUID
      ELSE
         NUA=NU-1
      ENDIF
C POCET DELICICH INTERVALU
      IF(NUA.LE.4)THEN
         NX=NIX
         NY=NIY
         NS=1
      ELSE
         NX=2*NIX
         NY=2*NIY
         NS=2
      ENDIF
C KONTROLA ZADAVANYCH PARAMETRU
      IF(NUA.LT.3.OR.NUA.GT.8.OR.N8.LT.4.OR.N8.GT.8.OR.
     /   NX.LT.1.OR.NY.LT.1.OR.NX.GE.MAXU.OR.NY.GE.MAXU)RETURN
C VYPOCET VEKTORU BEZROZMERNYCH SOURADNIC DELICICH BODU
      CALL FEMDIV(-1.,1.,XA,NX,AX)
      CALL FEMDIV(-1.,1.,YA,NY,AY)
      N=NDOLD
C GENEROVANI SOURADNIC UZLU (ISOPARAMETRICKE FUNKCE)
C A INDEXU UZLOVYCH PARAMETRU LPU.
C NEGENERUJI SE UZLY V TEZISTICH ELEMENTU (ANI KDYZ NUID<0)
C---------------------------------------------------------
      YODD=.FALSE.
      DO IY=1,NY+1
       YODD=.NOT.YODD
       XODD=.FALSE.
       DO IX=1,NX+1
         XODD=.NOT.XODD
C VYNECHAVAJI SE BODY PRO SUDE IX I IY POKUD NU=8
         IF(NUA.NE.8.OR.XODD.OR.YODD)THEN
           N=N+1
           IF(N.GT.MAXND)THEN
              WRITE(*,*)' MAX.NODE=',MAXND
              RETURN
           ENDIF
           X(N)=0
           Y(N)=0
           DO I=1,N8
             ZX=XA(IX)*RX(I)
             ZY=YA(IY)*RY(I)
             IF(N8.EQ.4)THEN
                BI=(1+ZX)*(1+ZY)/4.
             ELSEIF(I.LE.4)THEN
                BI=(1+ZX)*(1+ZY)*(ZX+ZY-1)/4.
             ELSEIF(I.EQ.5.OR.I.EQ.7)THEN
                BI=(1-XA(IX)**2)*(1+ZY)/2.
             ELSE
                BI=(1-YA(IY)**2)*(1+ZX)/2.
             ENDIF
             X(N)=X(N)+X8(I)*BI
             Y(N)=Y(N)+Y8(I)*BI
           ENDDO
C STANOVENI POCTU PARAMETRU UZLOVEHO BODU.
C KDYZ JE NUA<=4 JE  TO VZDY NPU(1),
C POKUD JE NUA>4 MUZE TO BYT NPU(1) (PRO LICHE IX,IY), JINAK NPU(2)
           IF(NUA.LE.4.OR.(XODD.AND.YODD))THEN
             KINDU(N)=1
           ELSE
             KINDU(N)=2
           ENDIF
         ENDIF
       ENDDO
      ENDDO
C GENEROVANI MATICE KONEKTIVITY
C-------------------------------
      NE=NEOLD
C LB-INDEX BOTTOM, LM-MIDDLE, LT-TOP
      IF(NUA.LE.4)THEN
         NBOT=NX+1
         NMID=0
      ELSEIF(NUA.EQ.6)THEN
         NBOT=NX+1
         NMID=NX+1
      ELSE
         NBOT=NX+1
         NMID=NIX+1
      ENDIF
      IBLOCK=0
      DO IY=1,NY,NS
         LB=IBLOCK*(NBOT+NMID)+NDOLD
         LM=LB+NBOT
         LT=LM+NMID
         IBLOCK=IBLOCK+1
         DO IX=1,NX,NS
            SELECT CASE(NUA)
            CASE(4)
C CTYRUHELNIKOVE ELEMENTY SE 4 UZLY
               NE=NE+1
               L0=LUE(NE)
               IUE(L0+1)=LB+IX
               IUE(L0+2)=LB+IX+NS
               IUE(L0+3)=LT+IX+NS
               IUE(L0+4)=LT+IX
               LUE(NE+1)=L0+NU
            CASE(3)
C TROJUHELNIKOVE ELEMENTY SE 3 UZLY
               NE=NE+1
               L0=LUE(NE)
               IUE(L0+1)=LB+IX
               IUE(L0+2)=LB+IX+1
               IUE(L0+3)=LT+IX
               LUE(NE+1)=L0+NU
               NE=NE+1
               L0=LUE(NE)
               IUE(L0+1)=LB+IX+1
               IUE(L0+2)=LT+IX+1
               IUE(L0+3)=LT+IX
               LUE(NE+1)=L0+NU
             CASE(6)
C TROJUHELNIKOVE ELEMENTY SE 6 UZLY
               NE=NE+1
               L0=LUE(NE)
               IUE(L0+1)=LB+IX
               IUE(L0+2)=LB+IX+2
               IUE(L0+3)=LT+IX
               IUE(L0+4)=LB+IX+1
               IUE(L0+5)=LM+IX+1
               IUE(L0+6)=LM+IX
               LUE(NE+1)=L0+NU
               NE=NE+1
               L0=LUE(NE)
               IUE(L0+1)=LB+IX+2
               IUE(L0+2)=LT+IX+2
               IUE(L0+3)=LT+IX
               IUE(L0+4)=LM+IX+2
               IUE(L0+5)=LT+IX+1
               IUE(L0+6)=LM+IX+1
               LUE(NE+1)=L0+NU
             CASE(8)
C CTYRUHELNIKOVE ELEMENTY S 8 UZLY
               NE=NE+1
               L0=LUE(NE)
               IUE(L0+1)=LB+IX
               IUE(L0+2)=LB+IX+2
               IUE(L0+3)=LT+IX+2
               IUE(L0+4)=LT+IX
               IUE(L0+5)=LB+IX+1
               IUE(L0+6)=LM+2
               IUE(L0+7)=LT+IX+1
               IUE(L0+8)=LM+1
               LUE(NE+1)=L0+NU
               LM=LM+1
          ENDSELECT
          IF(NE.GE.MAXEL)THEN
              WRITE(*,*)' MAX.ELEMENTS=',MAXEL
              RETURN
          ENDIF
        ENDDO
      ENDDO
      NENEW=NE
C
C GENEROVANI UZLU V TEZISTICH ELEMENTU
C
      DO IE=NEOLD+1,NENEW
        MUE(IE)=NUID
        IF(NUID.LT.0)THEN
          NUE=LUE(IE+1)-LUE(IE)-1
          L0=LUE(IE)
C TEZISTE ELEMENTU IE
          XG=0
          YG=0
          DO I=1,NUE
            IU=IUE(L0+I)
            XG=XG+X(IU)
            YG=YG+Y(IU)
          ENDDO
          N=N+1
          IF(N.GT.MAXND)THEN
             WRITE(*,*)' MAX.NODE=',MAXND
             RETURN
          ENDIF
          X(N)=XG/NUE
          Y(N)=YG/NUE
C DOPLNENI INDEXU UZLU N DO VEKTORU KONEKTIVITY
          IUE(L0+NUE+1)=N
C DOPLNENI INDEXU PARAMETRU
          KINDU(N)=3
        ENDIF
      ENDDO
      NDNEW=N
      END
 
